from typing import List, Optional, Tuple
from ..models.department import Department
from tortoise.expressions import Q


class DepartmentService:
    def __init__(self):
        pass

    async def get_children_ids(self, pid: int) -> List[int]:
        departments = await Department.filter(pid=pid, status=1).all()
        if not departments:
            return []

        ids = []
        for department in departments:
            children_ids = await self.get_children_ids(department.id)
            if children_ids:
                ids.extend(children_ids)
            ids.append(department.id)
        return ids

    async def get_children_departments(self, pid: int) -> List[Department]:
        departments = await Department.filter(pid=pid, status=1).all()
        if not departments:
            return []

        children_departments = []
        for department in departments:
            children = await self.get_children_departments(department.id)
            if children:
                children_departments.extend(children)
            children_departments.append(department)
        return children_departments

    async def get_info_by_id(
        self, department_id: int
    ) -> Tuple[Optional[Department], Optional[Exception]]:
        department = await Department.filter(status=1, id=department_id).first()
        if department:
            return department
        return None

    async def get_list(self) -> List[Department]:
        departments = await Department.filter(status=1).all()
        return departments

    async def get_list_by_ids(self, ids: List[int]) -> List[Department]:
        departments = await Department.filter(Q(id__in=ids), status=1).all()
        return departments



from typing import Any, Dict, List, Optional

from pydantic import model_validator

from ...component import Component
from .base import Base


class FieldNames(Component):
    label: str = None  # 显示的标签文本
    value: str = None  # 选项的实际值
    children: str = None  # 子选项列表的字段名


class Option(Component):
    label: Optional[str] = None
    value: Any = None
    disabled: bool = None
    children: List["Option"] = None
    is_leaf: bool = None


class Cascader(Base):
    component: str = "cascaderField"
    """
    组件名称
    """

    allow_clear: bool = True
    """
    是否允许清除选择
    """

    auto_focus: bool = False
    """
    是否自动获取焦点
    """
    bordered: bool = True
    """
    是否有边框
    """

    clear_icon: Any = None
    """
    自定义清除图标
    """

    change_on_select: bool = False
    """
    点选每级菜单选项值都会发生变化
    """

    class_name: str = None
    """
    自定义类名
    """

    default_value: Any = None
    """
    默认选中项
    """

    disabled: Any = False
    """
    是否禁用
    """

    popup_class_name: str = None
    """
    浮层自定义类名
    """

    expand_icon: Any = None
    """
    次级菜单展开图标
    """

    expand_trigger: str = None
    """
    次级菜单展开方式（click 或 hover）
    """

    field_names: FieldNames = None
    """
    自定义 options 中 label、value、children 的字段
    """

    max_tag_count: int = None
    """
    最多显示多少个 tag
    """

    max_tag_placeholder: str = None
    """
    隐藏 tag 时显示的内容
    """

    max_tag_text_length: int = None
    """
    最大显示的 tag 文本长度
    """

    not_found_content: str = None
    """
    当下拉列表为空时显示的内容
    """

    open: bool = None
    """
    控制浮层显隐
    """

    options: List[Option] = []
    """
    可选项数据源
    """

    placeholder: str = None
    """
    输入框占位文本
    """

    placement: str = None
    """
    浮层预设位置
    """

    show_search: bool = None
    """
    是否在选择框中显示搜索框
    """

    size: str = None
    """
    输入框大小（large/middle/small）
    """

    status: str = None
    """
    校验状态（error/warning）
    """

    style: Dict[str, Any] = None
    """
    自定义样式
    """

    suffix_icon: Any = None
    """
    自定义选择框后缀图标
    """

    value: Any = None
    """
    当前选中值
    """

    multiple: bool = None
    """
    是否支持多选
    """

    show_checked_strategy: str = None
    """
    选中项回填策略
    """

    remove_icon: Any = None
    """
    自定义多选框清除图标
    """

    search_value: str = None
    """
    搜索值
    """

    dropdown_menu_column_style: Any = None
    """
    下拉菜单列的样式
    """

    @model_validator(mode="after")
    def init(self):
        self.set_key()
        self.set_width("400px")
        self.placeholder = "请选择"
        return self

    def get_options(self) -> List[Option]:
        """
        获取当前可选项。

        Returns:
            List[Option]: 可选项列表。
        """
        return self.options

    def set_options(self, *args: Any):
        """
        设置选项数据源。
        支持多种参数形式：
        - 单个选项列表
        - 使用 list 转换为选项
        - 指定 rootId、parentKeyName、labelName、valueName 构建树形结构
        """
        if len(args) == 1:
            if isinstance(args[0], list):
                self.options = args[0]
                return self
        elif len(args) >= 4:
            root_id = args[1] if len(args) > 1 else 0
            parent_key_name = args[2]
            label_name = args[3]
            value_name = args[4] if len(args) > 4 else "id"
            self.options = self.list_to_options(
                args[0], root_id, parent_key_name, label_name, value_name
            )
            return self
        return self

    def list_to_options(
        self,
        data: List[Dict[str, Any]],
        root_id: int,
        parent_key_name: str,
        label_name: str,
        value_name: str,
    ) -> List[Option]:
        """
        将扁平列表转换为级联选项树。
        """
        tree = []
        for item in data:
            pid = item.get(parent_key_name)
            if pid == root_id:
                children = self.build_tree(
                    data, item.get(value_name), parent_key_name, label_name, value_name
                )
                tree.append(
                    Option(
                        label=item[label_name],
                        value=item[value_name],
                        children=children,
                    )
                )
        return tree

    def build_tree(
        self,
        data: List[Dict[str, Any]],
        pid: int,
        parent_key_name: str,
        label_name: str,
        value_name: str,
    ) -> List[Option]:
        """
        递归构建树结构。
        """
        children = []
        for item in data:
            if item.get(parent_key_name) == pid:
                child = Option(
                    label=item[label_name],
                    value=item[value_name],
                    children=self.build_tree(
                        data, item[value_name], parent_key_name, label_name, value_name
                    ),
                )
                children.append(child)
        return children

    def set_allow_clear(self, allow_clear: bool):
        self.allow_clear = allow_clear
        return self

    def set_auto_focus(self, auto_focus: bool):
        self.auto_focus = auto_focus
        return self

    def set_bordered(self, bordered: bool):
        self.bordered = bordered
        return self

    def set_clear_icon(self, clear_icon: Any):
        self.clear_icon = clear_icon
        return self

    def set_change_on_select(self, change_on_select: bool):
        self.change_on_select = change_on_select
        return self

    def set_class_name(self, class_name: str):
        self.class_name = class_name
        return self

    def set_default_value(self, default_value: Any):
        self.default_value = default_value
        return self

    def set_popup_class_name(self, popup_class_name: str):
        self.popup_class_name = popup_class_name
        return self

    def set_expand_icon(self, expand_icon: Any):
        self.expand_icon = expand_icon
        return self

    def set_expand_trigger(self, expand_trigger: str):
        self.expand_trigger = expand_trigger
        return self

    def set_field_names(self, field_names: FieldNames):
        self.field_names = field_names
        return self

    def set_max_tag_count(self, max_tag_count: int):
        self.max_tag_count = max_tag_count
        return self

    def set_max_tag_placeholder(self, max_tag_placeholder: str):
        self.max_tag_placeholder = max_tag_placeholder
        return self

    def set_max_tag_text_length(self, max_tag_text_length: int):
        self.max_tag_text_length = max_tag_text_length
        return self

    def set_not_found_content(self, not_found_content: str):
        self.not_found_content = not_found_content
        return self

    def set_open(self, open: bool):
        self.open = open
        return self

    def set_placeholder(self, placeholder: str):
        self.placeholder = placeholder
        return self

    def set_placement(self, placement: str):
        self.placement = placement
        return self

    def set_show_search(self, show_search: bool):
        self.show_search = show_search
        return self

    def set_size(self, size: str):
        self.size = size
        return self

    def set_status(self, status: str):
        self.status = status
        return self

    def set_suffix_icon(self, suffix_icon: Any):
        self.suffix_icon = suffix_icon
        return self

    def set_value(self, value: Any):
        self.value = value
        return self

    def set_multiple(self, multiple: bool):
        self.multiple = multiple
        return self

    def set_show_checked_strategy(self, strategy: str):
        self.show_checked_strategy = strategy
        return self

    def set_remove_icon(self, icon: Any):
        self.remove_icon = icon
        return self

    def set_search_value(self, value: str):
        self.search_value = value
        return self

    def set_dropdown_menu_column_style(self, style: Any):
        self.dropdown_menu_column_style = style
        return self



import json
from datetime import datetime
from io import BytesIO

from openpyxl import Workbook
from openpyxl.utils import get_column_letter
from tortoise.queryset import QuerySet

from quark import Request, Response

from ...services.attachment import AttachmentService
from ..performs_queries import PerformsQueries
from ..resolves_actions import ResolvesActions


class ExportRequest:

    # 请求对象
    request: Request

    # 查询对象
    query: QuerySet

    # 列表页字段
    fields: list

    # 搜索项
    searches: list

    # 全局数据排序规则
    query_order: str

    # 列表页面的排序规则
    export_query_order: str

    def __init__(
        self,
        request: Request,
        query: QuerySet,
        query_order: str,
        export_query_order: str,
        fields: list,
        searches: list,
    ):
        self.request = request
        self.query = query
        self.query_order = query_order
        self.export_query_order = export_query_order
        self.fields = fields
        self.searches = searches

    def handle(self):
        """
        处理导出逻辑
        """
        data = self.query_data()
        fields = self.fields

        wb = Workbook()
        ws = wb.active
        ws.title = "Sheet1"

        # 写入表头
        for col_idx, field in enumerate(fields, start=1):
            label = field.get("label", "")
            ws[f"{get_column_letter(col_idx)}1"] = label

        # 写入数据行
        for row_idx, item in enumerate(data, start=2):
            for col_idx, field in enumerate(fields, start=1):
                name = field.get("name")
                component = field.get("component")

                value = None
                if component == "inputNumberField" or component == "textField":
                    value = item.get(name)
                elif component in ["selectField", "checkboxField", "radioField"]:
                    option_label = field.get("getOptionLabel")(item.get(name))
                    value = option_label
                elif component == "switchField":
                    option_label = field.get("getOptionLabel")(item.get(name))
                    value = option_label
                else:
                    value = item.get(name)

                # 特殊格式处理
                if component in ["datetimeField", "dateField"]:
                    format_str = field.get("format", "")
                    format_str = (
                        format_str.replace("YYYY", "%Y")
                        .replace("MM", "%m")
                        .replace("DD", "%d")
                        .replace("HH", "%H")
                        .replace("mm", "%M")
                        .replace("ss", "%S")
                    )
                    if isinstance(value, datetime):
                        value = value.strftime(format_str)

                ws[f"{get_column_letter(col_idx)}{row_idx}"] = value

        # 返回 Excel 文件流
        output = BytesIO()
        wb.save(output)
        output.seek(0)

        filename = f"data_{datetime.now().strftime('%Y%m%d%H%M%S')}.xlsx"
        headers = {
            "Content-Disposition": f"attachment; filename={filename}",
            "Content-Type": "application/octet-stream",
        }

        return Response(
            output.getvalue(),
            mimetype="application/vnd.openxmlformats-officedocument.spreadsheetml.sheet",
            headers=headers,
        )

    async def query_data(self):
        """
        查询数据
        """

        # 获取列过滤条件
        column_filters = self.column_filters()

        # 获取排序规则
        orderings = self.orderings()

        query = PerformsQueries(
            request=self.request,
            query=self.query,
        ).build_export_query(self.searches, column_filters)

        query = PerformsQueries(
            request=self.request,
            query_order=self.query_order,
            export_query_order=self.export_query_order,
        ).apply_index_orderings(query, orderings)

        result = query.all()

        return await self.performs_list(result)

    async def performs_list(self, items):
        """
        处理列表字段
        """
        result = []
        index_fields = self.fields
        for item in items:
            fields = {}

            for field in index_fields:
                component = field.component
                name = field.name

                if component == "actionField":
                    items_callback = field.callback
                    if items_callback:
                        action_items = await items_callback(item)
                    else:
                        action_items = field.items

                    rendered_actions = []
                    for action in action_items:
                        rendered_actions.append(
                            await ResolvesActions(self.request).build_action(action)
                        )

                    fields[name] = rendered_actions
                else:
                    callback = field.callback
                    if callback:
                        fields[name] = await callback(item)
                    else:
                        value = getattr(item, name, None)
                        if value is None:
                            continue

                        # JSON 字符串解析
                        if isinstance(value, str):
                            if value.startswith("[") or value.startswith("{"):
                                try:
                                    value = json.loads(value)
                                except:
                                    pass

                        # 图片字段处理
                        if component in ["imageField", "imagePickerField"]:
                            value = AttachmentService().get_image_url(value)

                        fields[name] = value

            result.append(fields)

        return result

    def column_filters(self):
        """
        获取列过滤条件
        """
        filter_str = self.request.query_params.get("filter")
        column_filters = {}
        try:
            column_filters = json.loads(filter_str)
        except:
            pass
        return column_filters

    def orderings(self):
        """
        获取排序规则
        """
        sorter_str = self.request.query_params.get("sorter")
        orderings = {}
        try:
            orderings = json.loads(sorter_str)
        except:
            pass
        return orderings



from fastapi import APIRouter, Request, Response
from fastapi.encoders import jsonable_encoder
from fastapi.responses import FileResponse, JSONResponse, StreamingResponse
from fastapi.staticfiles import StaticFiles

from . import cache, config, utils
from .message import Message
from .quark import Quark
from .storage import Storage
from .template.dashboard import Dashboard
from .template.auth import Auth
from .template.resource import Resource
from .template.upload import Upload



from typing import Any, List

from quark import Auth, Message, Request, cache
from quark.component.form import Rule, field
from quark.component.icon.icon import Icon
from quark.schemas import LoginData
from quark.services.auth import AuthService


class Index(Auth):

    async def init(self, request: Request):
        self.login_api = "/api/admin/auth/index/login"
        self.title = "QuarkPy"
        self.redirect = "/layout/index?api=/api/admin/dashboard/index/index"
        return self

    async def fields(self, request: Request) -> List[Any]:
        return [
            (
                field.Text(name="username")
                .set_placeholder("用户名")
                .set_rules([Rule.required("请输入用户名")])
                .set_width("100%")
                .set_size("large")
                .set_prefix(Icon().set_type("ant-design:user-outlined"))
            ),
            (
                field.Password(name="password")
                .set_placeholder("密码")
                .set_rules([Rule.required("请输入密码")])
                .set_width("100%")
                .set_size("large")
                .set_prefix(Icon().set_type("ant-design:lock-outlined"))
            ),
            (
                field.ImageCaptcha(name="captcha")
                .set_captcha_url("/api/admin/auth/index/captcha")
                .set_placeholder("验证码")
                .set_rules([Rule.required("请输入验证码")])
                .set_width("100%")
                .set_size("large")
                .set_prefix(Icon().set_type("ant-design:safety-certificate-outlined"))
            ),
        ]

    async def login(self, request: Request):
        data_json = await request.json()
        try:
            data = LoginData(**data_json)
        except Exception as e:
            return Message.error(str(e))

        # 验证验证码
        cached_captcha = await cache.get(data.captcha["uuid"])
        if cached_captcha != data.captcha["value"]:
            return Message.error("验证码错误")

        if not data.username or not data.password:
            return Message.error("用户名或密码不能为空")

        try:
            token = await AuthService(request).login(
                data.username, data.password, "admin"
            )
        except Exception as e:
            return Message.error(str(e))

        return Message.success("登录成功", {"token": token})



import json
from datetime import datetime

import openpyxl

from ..models.attachment import Attachment
from ..services.config import ConfigService


class AttachmentService:
    def __init__(self):
        self.config_service = ConfigService()

    async def get_list_by_search(
        self, admin_id, attachment_type, category_id, name, createtime, page
    ):
        query = Attachment.filter(status=1, source="ADMIN", uid=admin_id)

        if category_id:
            query = query.filter(category_id=category_id)
        if name:
            query = query.filter(name__contains=name)
        if len(createtime) == 2:
            query = query.filter(created_at__range=(createtime[0], createtime[1]))

        total = await query.count()
        attachments = await query.order_by("-id").offset((page - 1) * 8).limit(8)

        for attachment in attachments:
            attachment.url = (
                await self.get_url(attachment.url)
                + f"?timestamp={int(datetime.now().timestamp())}"
            )

        return attachments, total

    async def insert_get_id(self, attachment):
        await attachment.save()
        return attachment.id

    async def delete_by_id(self, attachment_id):
        return await Attachment.filter(id=attachment_id).delete()

    async def get_info_by_id(self, attachment_id):
        attachment = await Attachment.filter(status=1, id=attachment_id).first()
        return attachment

    async def update_by_id(self, attachment_id, data: Attachment):
        return await Attachment.filter(status=1, id=attachment_id).update(**data)

    async def get_info_by_hash(self, hash_value):
        return await Attachment.filter(status=1, hash=hash_value).first()

    async def get_url(self, *params):
        id = None
        attachment_type = None

        if len(params) == 1:
            id = params[0]
        elif len(params) == 2:
            attachment_type = params[0]
            id = params[1]

        if id is None or id == "":
            return ""

        http = ""
        path = ""
        web_site_domain = await self.config_service.get_value("WEB_SITE_DOMAIN")
        if web_site_domain is not None and web_site_domain != "":
            http = (
                "https://"
                if await self.config_service.get_value("SSL_OPEN") == "1"
                else "http://"
            )

        if isinstance(id, str):
            if "://" in id and "{" not in id:
                return id
            if "./" in id and "{" not in id:
                return http + web_site_domain + id.replace("./web/app/", "/", 1)
            if "/" in id and "{" not in id:
                return http + web_site_domain + id
            if "{" in id:
                try:
                    json_data = json.loads(id)
                    if isinstance(json_data, dict):
                        path = json_data.get("url", "")
                    elif isinstance(json_data, list):
                        path = json_data[0].get("url", "")
                except json.JSONDecodeError:
                    pass

            if "://" in path:
                return path
            if "./" in path:
                path = path.replace("./web/app/", "/", 1)
            if path:
                return http + web_site_domain + path

        attachment = await Attachment.filter(id=id, status=1).first()
        if attachment and attachment.id != 0:
            path = attachment.url
            if "://" in path:
                return path
            if "./" in path:
                path = path.replace("./web/app/", "/", 1)
            if path:
                return http + web_site_domain + path

        if attachment_type == "IMAGE":
            return http + web_site_domain + "/admin/default.png"
        return ""

    async def get_file_url(self, id):
        return await self.get_url("FILE", id)

    async def get_image_url(self, id):
        return await self.get_url("IMAGE", id)

    async def get_urls(self, id):
        paths = []
        http = (
            "https://"
            if await self.config_service.get_value("SSL_OPEN") == "1"
            else "http://"
        )
        web_site_domain = await self.config_service.get_value("WEB_SITE_DOMAIN")

        if isinstance(id, str):
            if "{" in id:
                try:
                    json_data = json.loads(id)
                    if isinstance(json_data, list):
                        for v in json_data:
                            path = v.get("url", "")
                            if "://" in path:
                                paths.append(path)
                            else:
                                if "./" in path:
                                    path = path.replace("./web/app/", "/", 1)
                                if path:
                                    path = http + web_site_domain + path
                                paths.append(path)
                except json.JSONDecodeError:
                    pass

        return paths

    async def get_path(self, id):
        path = ""
        if isinstance(id, str):
            if "://" in id and "{" not in id:
                return id
            if "./" in id and "{" not in id:
                return id
            if "/" in id and "{" not in id:
                return id
            if "{" in id:
                try:
                    json_data = json.loads(id)
                    if isinstance(json_data, dict):
                        id = json_data.get("id", "")
                    elif isinstance(json_data, list):
                        id = json_data[0].get("id", "")
                except json.JSONDecodeError:
                    pass

        attachment = await Attachment.filter(id=id, status=1).first()
        return attachment.path if attachment else ""

    async def get_file_path(self, id):
        return await self.get_path(id)

    async def get_image_path(self, id):
        return await self.get_path(id)

    async def get_paths(self, id):
        paths = []
        if isinstance(id, str):
            if "{" in id:
                try:
                    json_data = json.loads(id)
                    if isinstance(json_data, list):
                        for v in json_data:
                            paths.append(await self.get_path(v["id"]))
                except json.JSONDecodeError:
                    pass
        return paths

    async def get_excel_data(self, file_id):
        file = await Attachment.filter(id=file_id, status=1).first()
        if not file:
            return []

        workbook = openpyxl.load_workbook(file.path)
        sheet = workbook.active
        data = []
        for row in sheet.iter_rows(values_only=True):
            data.append(row)
        return data

    async def count_by_type(self, file_type):
        return await Attachment.filter(type=file_type).count()



from quark import APIRouter, JSONResponse, Request, jsonable_encoder

from .. import loader

router = APIRouter(prefix="/api/admin", tags=["文件上传"])


@router.get("/upload/{resource}/getList")
async def get_list(request: Request, resource: str):
    res = await loader.load_resource_object(request, resource, "Upload")
    return JSONResponse(
        content=jsonable_encoder(await res.get_list(request), exclude_none=True)
    )


@router.get("/upload/{resource}/delete")
async def delete_get(request: Request, resource: str):
    res = await loader.load_resource_object(request, resource, "Upload")
    return JSONResponse(
        content=jsonable_encoder(await res.delete(request), exclude_none=True)
    )


@router.post("/upload/{resource}/delete")
async def delete_post(request: Request, resource: str):
    res = await loader.load_resource_object(request, resource, "Upload")
    return JSONResponse(
        content=jsonable_encoder(await res.delete(request), exclude_none=True)
    )


@router.post("/upload/{resource}/crop")
async def crop(request: Request, resource: str):
    res = await loader.load_resource_object(request, resource, "Upload")
    return JSONResponse(
        content=jsonable_encoder(await res.crop(request), exclude_none=True)
    )


@router.post("/upload/{resource}/handle")
async def handle(request: Request, resource: str):
    res = await loader.load_resource_object(request, resource, "Upload")
    return JSONResponse(
        content=jsonable_encoder(await res.handle(request), exclude_none=True)
    )


@router.post("/upload/{resource}/base64Handle")
async def base64_handle(request: Request, resource: str):
    res = await loader.load_resource_object(request, resource, "Upload")
    return JSONResponse(
        content=jsonable_encoder(await res.base64_handle(request), exclude_none=True)
    )



from typing import Any, Dict, List

from quark import Request

from ..utils import list_to_tree
from .resolves_actions import ResolvesActions
from .resolves_fields import ResolvesFields
from .resolves_searches import ResolvesSearches


class ResourceIndex:
    """资源：增删改查"""

    async def index_table_list_to_tree(
        self, request: Request, list_data: List[Any]
    ) -> List[Any]:
        """
        列表页数据转换成树结构
        """
        data = request.query_params
        if isinstance(data.get("search"), dict) and data["search"]:
            return list_data

        pk_name = "id"
        pid_name = "pid"
        children_name = "children"
        root_id = 0

        table_list_to_tree = self.table_list_to_tree
        if isinstance(table_list_to_tree, bool):
            if not table_list_to_tree:
                return list_data
        elif isinstance(table_list_to_tree, dict):
            pk_name = table_list_to_tree.get("pkName", "id")
            pid_name = table_list_to_tree.get("pidName", "pid")
            children_name = table_list_to_tree.get("childrenName", "children")
            root_id = table_list_to_tree.get("rootId", 0)

        tree = list_to_tree(list_data, pk_name, pid_name, children_name, root_id)
        return tree

    async def index_table_extra_render(self, request: Request) -> Any:
        """
        列表页表格主体
        """
        return None

    async def index_table_tool_bar(self, request: Request) -> Any:
        """
        列表页工具栏组件
        """
        actions = await self.actions(request)
        index_table_actions = await ResolvesActions(
            request=request, actions=actions
        ).index_table_actions()
        index_table_title = await self.index_table_title(request)
        index_table_menu = await self.index_table_menu(request)
        return (
            self.table_tool_bar.set_title(index_table_title)
            .set_actions(index_table_actions)
            .set_menu(index_table_menu)
        )

    async def index_table_tree_bar(self, request: Request) -> Any:
        """
        列表页树形结构组件
        """
        return self.table_tree_bar

    async def index_table_title(self, request: Request) -> str:
        """
        获取列表标题
        """
        return self.title + self.table_title_suffix

    async def index_table_menu_items(self, request: Request) -> List[Dict[str, str]]:
        return await self.menu_items(request)

    async def index_table_menu(self, request: Request) -> Dict[str, Any]:
        # 获取菜单项
        items = await self.index_table_menu_items(request)
        if items is None:
            return {}

        # 返回表格菜单
        return {"type": "tab", "items": items}

    async def index_component_render(self, request: Request, data: Any) -> Any:
        """
        列表页组件渲染主逻辑
        """
        table_title = await self.index_table_title(request)
        table_extra_render = await self.index_table_extra_render(request)
        table_tool_bar = await self.index_table_tool_bar(request)
        table_tree_bar = await self.index_table_tree_bar(request)
        fields = await self.fields(request)
        actions = await self.actions(request)
        searches = await self.searches(request)
        index_table_row_actions = await ResolvesActions(
            request=request, actions=actions
        ).index_table_row_actions()
        table_columns = ResolvesFields(
            request=request,
            fields=fields,
            table_column=self.table_column,
            table_row_actions=index_table_row_actions,
            table_action_column_title=self.table_action_column_title,
            table_action_column_width=self.table_action_column_width,
        ).index_table_columns()
        index_searches = ResolvesSearches(
            request=request, search=self.table_search, searches=searches
        ).index_searches()

        # 是否开启树形表格
        if self.table_list_to_tree is not None:
            data = await self.index_table_list_to_tree(request, data)

        # 构建表格配置
        table = (
            self.table.set_polling(self.table_polling)
            .set_title(table_title)
            .set_table_extra_render(table_extra_render)
            .set_tool_bar(table_tool_bar)
            .set_tree_bar(table_tree_bar)
            .set_columns(table_columns)
            .set_searches(index_searches)
        )

        page_size = self.page_size
        if page_size is None:
            return table.set_datasource(data)

        # 如果不是整数分页配置，直接返回数据
        if not isinstance(page_size, int) or isinstance(page_size, bool):
            return table.set_datasource(data)
        else:
            current = data.get("current")
            page_size_val = data.get("pageSize")
            page_size_options = data.get("pageSizeOptions")
            total = data.get("total")
            items = data.get("items")

            return table.set_pagination(
                current, page_size_val, int(total), 1, page_size_options
            ).set_datasource(items)

    async def before_index_showing(
        self, request: Request, list: List[Dict[str, Any]]
    ) -> List[Any]:
        """
        页面显示前回调
        """
        return list



from typing import Any, List, Optional

from quark import Request

from ..component.action.action import Action
from ..component.dropdown.dropdown import Dropdown
from ..component.form.form import Form


class ResolvesActions:

    # 请求
    request: Optional[Request] = None

    # 行为
    actions: Optional[List[Any]] = None

    def __init__(self, request: Request, actions: Optional[List[Any]] = None):
        self.request = request
        self.actions = actions

    async def index_table_actions(self) -> List[Any]:
        """获取列表页顶部工具栏中的动作"""
        items = []
        if self.actions is not None:
            for action in self.actions:
                if action.shown_on_index():
                    items.append(await self.build_action(action))

        return items

    async def index_table_row_actions(self) -> List[Any]:
        """获取列表页行内动作"""
        items = []
        if self.actions is not None:
            for action in self.actions:
                if action.shown_on_index_table_row():
                    items.append(await self.build_action(action))

        return items

    async def form_actions(self) -> List[Any]:
        """获取表单页动作"""
        items = []
        if self.actions is not None:
            for action in self.actions:
                if action.shown_on_form():
                    items.append(await self.build_action(action))

        return items

    async def form_extra_actions(self) -> List[Any]:
        """表单页右上角自定义区域行为"""
        items = []
        if self.actions is not None:
            for action in self.actions:
                if action.shown_on_form_extra():
                    items.append(await self.build_action(action))

        return items

    async def detail_actions(self) -> List[Any]:
        """获取详情页动作"""
        items = []
        if self.actions is not None:
            for action in self.actions:
                if action.shown_on_detail():
                    items.append(await self.build_action(action))

        return items

    async def detail_extra_actions(self) -> List[Any]:
        """详情页右上角自定义区域行为"""
        items = []
        if self.actions is not None:
            for action in self.actions:
                if action.shown_on_detail_extra():
                    items.append(await self.build_action(action))

        return items

    async def build_action(self, item) -> Any:
        """构建行为组件（支持 link/modal/drawer/dropdown/switch 等）"""
        name = item.get_name()
        with_loading = item.get_with_loading()
        reload = item.get_reload()
        uri_key = item.get_uri_key(item)
        api = item.get_api()
        params = item.get_api_params()
        action_type = item.get_action_type()
        button_type = item.get_type()
        size = item.get_size()
        icon = item.get_icon()
        block = item.get_block()
        batch = item.get_batch()
        disabled = item.get_disabled()
        danger = item.get_danger()
        ghost = item.get_ghost()
        confirm_title = item.get_confirm_title()
        confirm_text = item.get_confirm_text()
        confirm_type = item.get_confirm_type()

        if not api:
            api = self.build_action_api(params, uri_key)

        action_component = (
            Action().set_label(name).set_with_loading(with_loading).set_api(api)
        )

        if reload:
            action_component.set_reload(reload)

        if action_type:
            action_component.set_action_type(action_type)

        if button_type:
            action_component.set_type(button_type, False)

        if size:
            action_component.set_size(size)

        if icon:
            action_component.set_icon(icon)

        if block:
            action_component.set_block(block)

        if batch:
            action_component.set_batch(batch)

        if disabled:
            action_component.set_disabled(disabled)

        if danger:
            action_component.set_danger(danger)

        if ghost:
            action_component.set_ghost(ghost)

        # 特定行为类型渲染
        if action_type == "link":
            href = await item.get_href(self.request)
            target = await item.get_target(self.request)
            action_component.set_link(href, target)

        elif action_type == "modal":
            init_api = self.build_init_api(params, uri_key)
            width = item.get_width()
            destroy_on_close = item.get_destroy_on_close()
            body = await item.get_body(self.request)
            actions = await item.get_actions(self.request)

            action_component.set_modal(
                lambda m: m.set_title(name)
                .set_init_api(init_api)
                .set_width(width)
                .set_body(body)
                .set_actions(actions)
                .set_destroy_on_close(destroy_on_close)
            )

        elif action_type == "drawer":
            init_api = self.build_init_api(params, uri_key)
            width = item.get_width()
            destroy_on_close = item.get_destroy_on_close()
            body = await item.get_body(self.request)
            actions = await item.get_actions(self.request)

            action_component.set_drawer(
                lambda d: d.set_title(name)
                .set_init_api(init_api)
                .set_width(width)
                .set_body(body)
                .set_actions(actions)
                .set_destroy_on_close(destroy_on_close)
            )

        elif action_type == "dropdown":
            overlay = await item.get_menu(self.request)
            overlay_style = item.get_overlay_style()
            placement = item.get_placement()
            trigger = item.get_trigger()
            arrow = item.get_arrow()

            action_component = (
                Dropdown()
                .set_label(name)
                .set_menu(overlay)
                .set_overlay_style(overlay_style)
                .set_placement(placement)
                .set_trigger(trigger)
                .set_arrow(arrow)
                .set_size(size)
                .set_type(button_type)
            )

            if icon:
                action_component.set_icon(icon)

        elif action_type == "switch":
            checked_children = item.get_checked_children()
            unchecked_children = item.get_unchecked_children()
            field_name = item.get_field_name()
            field_value = item.get_field_value()

            action_component.set_field_name(field_name).set_field_value(
                field_value
            ).set_checked_children(checked_children).set_un_checked_children(
                unchecked_children
            )

        # 只对 Action 类型的组件设置确认对话框
        if confirm_title and isinstance(action_component, Action):
            action_component.set_with_confirm(confirm_title, confirm_text, confirm_type)

        return action_component

    def build_action_api(self, params: List[str], uri_key: str) -> str:
        """构建行为 API 地址"""
        if self.request is None:
            return ""

        api = self.request.url.path
        api_paths = api.split("/")

        new_api = api
        if len(api_paths) <= 2:
            return ""

        last_segment = api_paths[-1]
        if last_segment == "index":
            new_api = api.replace("/index", f"/action/{uri_key}", 1)
        elif last_segment == "create":
            new_api = api.replace("/create", f"/action/{uri_key}", 1)
        elif last_segment == "edit":
            new_api = api.replace("/edit", f"/action/{uri_key}", 1)
        elif last_segment == "detail":
            new_api = api.replace("/detail", f"/action/{uri_key}", 1)
        elif last_segment == "form":
            last_path = api_paths[-2]
            new_api = api.replace(f"{last_path}/form", f"action/{uri_key}", 1)

        # 添加查询参数
        params_uri = ""
        for param in params:
            params_uri += f"{param}=${{{param}}}&"

        if params_uri:
            new_api = f"{new_api}?{params_uri.rstrip('&')}"

        return new_api

    def build_init_api(self, params: List[str], uri_key: str) -> str:
        """构建初始化数据 API 地址"""
        if self.request is None:
            return ""

        api = self.request.url.path
        api_paths = api.split("/")

        new_api = api
        if len(api_paths) <= 2:
            return ""

        last_segment = api_paths[-1]
        if last_segment == "index":
            new_api = api.replace("/index", f"/action/{uri_key}/values", 1)
        elif last_segment == "create":
            new_api = api.replace("/create", f"/action/{uri_key}/values", 1)
        elif last_segment == "edit":
            new_api = api.replace("/edit", f"/action/{uri_key}/values", 1)
        elif last_segment == "detail":
            new_api = api.replace("/detail", f"/action/{uri_key}/values", 1)
        elif last_segment == "form":
            last_path = api_paths[-2]
            new_api = api.replace(f"{last_path}/form", f"action/{uri_key}/values", 1)

        # 添加查询参数
        params_uri = ""
        for param in params:
            params_uri += f"{param}=${{{param}}}&"

        if params_uri:
            new_api = f"{new_api}?{params_uri.rstrip('&')}"

        return new_api



from typing import Any
from ..component import Component

class Line(Component):
    """折线图表类"""
    component: str = "line"
    api: str = ""
    width: int = 0
    height: int = 0
    auto_fit: bool = False
    padding: Any = None
    append_padding: Any = None
    renderer: str = ""
    limit_in_plot: bool = False
    locale: str = ""
    data: Any = None
    x_field: str = ""
    y_field: str = ""
    meta: Any = None
    smooth: bool = False
    key: str = ""
    crypt: bool = False

    # 数据接口
    def set_api(self, api: str) -> 'Line':
        self.api = api
        return self

    # 设置图表宽度
    def set_width(self, width: int) -> 'Line':
        self.width = width
        return self

    # 设置图表高度
    def set_height(self, height: int) -> 'Line':
        self.height = height
        return self

    # 图表是否自适应容器宽高。当 autoFit 设置为 true 时，width 和 height 的设置将失效。
    def set_auto_fit(self, auto_fit: bool) -> 'Line':
        self.auto_fit = auto_fit
        return self

    # 画布的 padding 值，代表图表在上右下左的间距，可以为单个数字 16，或者数组 [16, 8, 16, 8] 代表四个方向，或者开启 auto，由底层自动计算间距。
    def set_padding(self, padding: Any) -> 'Line':
        self.padding = padding
        return self

    # 额外增加的 appendPadding 值，在 padding 的基础上，设置额外的 padding 数值，可以是单个数字 16，或者数组 [16, 8, 16, 8] 代表四个方向。
    def set_append_padding(self, append_padding: Any) -> 'Line':
        self.append_padding = append_padding
        return self

    # 设置图表渲染方式为 canvas 或 svg。
    def set_renderer(self, renderer: str) -> 'Line':
        self.renderer = renderer
        return self

    # 是否对超出坐标系范围的 Geometry 进行剪切。
    def set_limit_in_plot(self, limit_in_plot: bool) -> 'Line':
        self.limit_in_plot = limit_in_plot
        return self

    # 指定具体语言，目前内置 'zh-CN' and 'en-US' 两个语言，你也可以使用 G2Plot.registerLocale 方法注册新的语言。语言包格式参考：src/locales/en_US.ts
    def set_locale(self, locale: str) -> 'Line':
        self.locale = locale
        return self

    # 数据
    def set_data(self, data: Any) -> 'Line':
        self.data = data
        return self

    # X轴字段
    def set_x_field(self, x_field: str) -> 'Line':
        self.x_field = x_field
        return self

    # y轴字段
    def set_y_field(self, y_field: str) -> 'Line':
        self.y_field = y_field
        return self

    # 通过 meta 可以全局化配置图表数据元信息，以字段为单位进行配置。在 meta 上的配置将同时影响所有组件的文本信息。传入以字段名为 key，MetaOption 为 value 的配置，同时设置多个字段的元信息。
    def set_meta(self, meta: Any) -> 'Line':
        self.meta = meta
        return self

    # 是否平滑
    def set_smooth(self, smooth: bool) -> 'Line':
        self.smooth = smooth
        return self

    # 初始化
    def init(self) -> 'Line':
        self.component = "line"
        self.set_key("DEFAULT_KEY", False)
        return self

    def set_key(self, key: str, crypt: bool) -> 'Line':
        self.key = key
        self.crypt = crypt
        return self


from pydantic import model_validator
from .base import Base


class Fieldset(Base):

    component: str = "fieldsetField"

    @model_validator(mode="after")
    def init(self):
        self.only_on_forms()
        self.set_key()
        return self



from tortoise import fields
from tortoise.models import Model


class Attachment(Model):
    id = fields.IntField(pk=True)
    uid = fields.IntField(default=0)
    source = fields.CharField(max_length=255, null=True)
    category_id = fields.IntField(default=0)
    name = fields.CharField(max_length=255)
    type = fields.CharField(max_length=255, null=True)
    sort = fields.IntField(default=0)
    size = fields.BigIntField(default=0)
    ext = fields.CharField(max_length=255, null=True)
    path = fields.CharField(max_length=255)
    url = fields.CharField(max_length=255)
    hash = fields.CharField(max_length=255)
    extra = fields.CharField(max_length=5000, default="", null=False)
    status = fields.IntField(default=1)
    created_at = fields.DatetimeField(auto_now_add=True)
    updated_at = fields.DatetimeField(auto_now=True, null=True)

    class Meta(Model.Meta):
        table = "attachments"



from pydantic import model_validator
from typing import Any, Dict
from ..component import Component

class TabPane(Component):
    title: str = None
    body: Any = None
    component: str = "tabPane"

    @model_validator(mode="after")
    def init(self):
        self.set_key()
        return self

    # 设置方法（链式调用）
    def set_style(self, style: Dict[str, Any]):
        self.style = style
        return self

    def set_title(self, title: str):
        self.title = title
        return self

    def set_body(self, body: Any):
        self.body = body
        return self


import json
from typing import Any, List

from pydantic import model_validator

from quark.component.form.fields.when import Item, When

from .base import Base


class Dependency(Base):

    component: str = "dependencyField"
    """
    组件名称
    """

    ignore_form_list_field: bool = False
    """
    是否忽略 FormList 上的字段，默为 False
    """

    names: List[str] = []
    """
    组件内容
    """

    @model_validator(mode="after")
    def init(self):
        self.only_on_forms()
        return self

    def set_when(self, *value: Any):
        """
        用法：
        set_when("name", 1, lambda: [field1, field2])
        set_when("name", ">", 1, lambda: [field1, field2])
        """
        w = When()
        i = Item(body=None)

        if len(value) == 3:
            # 默认 operator =
            name, option, callback = value
            operator = "="
        elif len(value) == 4:
            name, operator, option, callback = value
        else:
            raise ValueError("set_when 参数数量必须是3或4个")

        # 执行回调生成 body
        if callable(callback):
            i.body = callback()

        # 生成条件表达式
        get_option = str(option)
        if operator == "!=":
            i.condition = f"<%=String({name}) !== '{get_option}' %>"
        elif operator == "=":
            i.condition = f"<%=String({name}) === '{get_option}' %>"
        elif operator == ">":
            i.condition = f"<%=String({name}) > '{get_option}' %>"
        elif operator == "<":
            i.condition = f"<%=String({name}) < '{get_option}' %>"
        elif operator == "<=":
            i.condition = f"<%=String({name}) <= '{get_option}' %>"
        elif operator == ">=":
            i.condition = f"<%=String({name}) >= '{get_option}' %>"
        elif operator == "has":
            i.condition = f"<%=(String({name}).indexOf('{get_option}') !=-1) %>"
        elif operator == "in":
            json_str = json.dumps(option)
            i.condition = f"<%=({json_str}.indexOf({name}) !=-1) %>"
        else:
            i.condition = f"<%=String({name}) === '{get_option}' %>"

        i.condition_name = name
        i.condition_operator = operator
        i.option = option

        self.when_item.append(i)
        self.when = w.set_items(self.when_item)
        self.names.append(name)

        return self

    def set_ignore_form_list_field(self, ignore_form_list_field: bool):
        """
        忽略 FormList 上的字段。

        Args:
            ignore_form_list_field (bool): 是否忽略。

        Returns:
            Component: 当前组件实例，支持链式调用。
        """
        self.ignore_form_list_field = ignore_form_list_field
        return self



from pydantic import Field, model_validator
from typing import Any, Dict, Optional
from ..component import Component

class Card(Component):
    component: str = Field(default="card")
    title: str = Field("", description="标题文字")
    sub_title: str = Field("", description="二级标题文字")
    tip: str = Field("", description="标题右侧图标 hover 提示信息")
    extra: Optional[Any] = Field(None, description="右上角自定义区域")
    layout: str = Field("default", description="内容布局，支持垂直居中 default | center")
    loading: bool = Field(False, description="加载中，支持自定义 loading 样式")
    col_span: Optional[Any] = Field(24, description="栅格布局宽度，24 栅格，支持指定宽度 px 或百分比, 支持响应式的对象写法 { xs: 8, sm: 16, md: 24}")
    gutter: Optional[Any] = Field(0, description="栅格布局宽度，24 栅格，支持指定宽度 px 或百分比, 支持响应式的对象写法 { xs: 8, sm: 16, md: 24}")
    split: str = Field("", description="拆分卡片的方向, vertical | horizontal")
    bordered: bool = Field(True, description="是否有边框")
    ghost: bool = Field(False, description="幽灵模式，即是否取消卡片内容区域的 padding 和 卡片的背景颜色")
    header_bordered: bool = Field(True, description="页头是否有分割线")
    collapsible: bool = Field(False, description="页头是否有分割线")
    default_collapsed: bool = Field(False, description="默认折叠, 受控时无效")
    body: Optional[Any] = Field(None, description="卡牌内容")
    style: Optional[Dict[str, Any]] = Field(None, description="样式")

    @model_validator(mode="after")
    def init(self):
        self.set_key("action")
        return self
    
    def set_style(self, style: Dict[str, Any]):
        # 设置样式
        self.style = style
        return self

    def set_title(self, title: str):
        # 设置标题文字
        self.title = title
        return self

    def set_sub_title(self, sub_title: str):
        # 设置二级标题文字
        self.sub_title = sub_title
        return self

    def set_tip(self, tip: str):
        # 设置标题右侧图标 hover 提示信息
        self.tip = tip
        return self

    def set_extra(self, extra: Any):
        # 设置右上角自定义区域
        self.extra = extra
        return self

    def set_layout(self, layout: str):
        # 设置内容布局
        self.layout = layout
        return self

    def set_loading(self, loading: bool):
        # 设置加载中状态
        self.loading = loading
        return self

    def set_col_span(self, col_span: Any):
        # 设置栅格布局宽度
        self.col_span = col_span
        return self

    def set_gutter(self, gutter: Any):
        # 设置栅格布局宽度
        self.gutter = gutter
        return self

    def set_split(self, split: str):
        # 设置拆分卡片的方向
        self.split = split
        return self

    def set_bordered(self, bordered: bool):
        # 设置是否有边框
        self.bordered = bordered
        return self

    def set_ghost(self, ghost: bool):
        # 设置幽灵模式
        self.ghost = ghost
        return self

    def set_header_bordered(self, header_bordered: bool):
        # 设置页头是否有分割线
        self.header_bordered = header_bordered
        return self

    def set_collapsible(self, collapsible: bool):
        # 设置页头是否有分割线
        self.collapsible = collapsible
        return self

    def set_default_collapsed(self, default_collapsed: bool):
        # 设置默认折叠状态
        self.default_collapsed = default_collapsed
        return self

    def set_body(self, body: Any):
        # 设置卡牌内容
        self.body = body
        return self


from quark.component.descriptions.fields.text import Text
from quark.template.metric.descriptions import Descriptions


class TeamInfo(Descriptions):
    title: str = "团队信息"
    col: int = 12

    async def calculate(self):
        return self.result(
            [
                Text().set_label("作者").set_value("tangtanglove"),
                Text().set_label("联系方式").set_value("dai_hang_love@126.com"),
                Text()
                .set_label("官方网址")
                .set_value(
                    "<a href='https://quarkcloud.io' target='_blank'>quarkcloud.io</a>"
                ),
                Text()
                .set_label("文档地址")
                .set_value(
                    "<a href='https://quarkcloud.io' target='_blank'>查看文档</a>"
                ),
                Text()
                .set_label("BUG反馈")
                .set_value(
                    "<a href='https://github.com/quarkcloudio/quark-py/issues' target='_blank'>提交BUG</a>"
                ),
                Text()
                .set_label("代码仓储")
                .set_value(
                    "<a href='https://github.com/quarkcloudio/quark-py' target='_blank'>Github</a>"
                ),
            ]
        )



from typing import Any

# 全局配置变量
config: dict[str, Any] = {}


def init(getConfig: dict[str, Any]) -> None:
    """初始化全局配置"""
    global config
    config = getConfig


def set(key: str, value: Any) -> None:
    config[key] = value


def get(key: str, default: Any = None) -> Any:
    return config.get(key, default)



import base64
import hashlib
import os
import uuid
from io import BytesIO
from typing import List

import filetype
from fastapi import UploadFile
from PIL import Image

from quark.schemas import FileInfo, MinioConfig, OSSConfig


class Storage:
    """
    文件系统类
    """

    # 文件对象
    file: UploadFile = None

    # Base64文件字符串
    file_base64_str: str = None

    # 文件字节流
    file_bytes: bytes = None

    # 上传限制
    limit_size: int = None

    # 允许的文件类型
    limit_type: List[str] = None

    # 限制图片宽度
    limit_image_width: int = None

    # 限制图片高度
    limit_image_height: int = None

    # 存储驱动
    driver: str = "local"

    # 是否上传图片时返回图片的额外信息
    with_image_extra = False

    # 是否随机文件名
    rand_name: bool = False

    # 保存路径
    save_path: str = ""

    # 保存文件名称
    save_name: str = ""

    # OSS配置
    oss_config: OSSConfig = None

    # Minio配置
    minio_config: MinioConfig = None

    def __init__(
        self,
        file: UploadFile = None,
        file_base64_str: str = None,
        file_bytes: bytes = None,
        limit_size: int = None,
        limit_type: list = None,
        limit_image_width: int = None,
        limit_image_height: int = None,
        rand_name: bool = None,
        save_path: str = None,
        save_name: str = None,
        oss_config: OSSConfig = None,
        minio_config: MinioConfig = None,
        driver: str = None,
        with_image_extra: bool = None,
    ):
        self.file = file
        self.file_base64_str = file_base64_str
        self.file_bytes = file_bytes
        self.driver = driver
        self.with_image_extra = with_image_extra
        self.oss_config = oss_config
        self.minio_config = minio_config
        self.limit_size = limit_size
        self.limit_type = limit_type
        self.limit_image_width = limit_image_width
        self.limit_image_height = limit_image_height
        self.rand_name = rand_name
        self.save_path = save_path
        self.save_name = save_name

    async def get_mime_type(self) -> str:
        if self.file is not None:
            return self.file.content_type
        elif self.file_base64_str is not None:
            kind = filetype.guess(base64.b64decode(self.file_base64_str))
            if kind is not None:
                return kind.mime
            else:
                return "application/octet-stream"
        elif self.file_bytes is not None:
            kind = filetype.guess(self.file_bytes)
            if kind is not None:
                return kind.mime
            else:
                return "application/octet-stream"
        else:
            return "application/octet-stream"

    async def get_ext(self) -> str:
        if self.file is not None:
            return os.path.splitext(self.file.filename)[1]
        elif self.file_base64_str is not None:
            kind = filetype.guess(base64.b64decode(self.file_base64_str))
            if kind is not None:
                return kind.extension
            else:
                return ""
        elif self.file_bytes is not None:
            kind = filetype.guess(self.file_bytes)
            if kind is not None:
                return kind.extension
            else:
                return ""
        else:
            return ""

    async def get_size(self) -> int:
        if self.file is not None:
            return self.file.size
        elif self.file_base64_str is not None:
            return len(base64.b64decode(self.file_base64_str))
        elif self.file_bytes is not None:
            return len(self.file_bytes)
        else:
            return 0

    async def get_bytes(self) -> bytes:
        # 如果已经有缓存，直接返回
        if self.file_bytes is not None:
            return self.file_bytes

        if self.file is not None:
            self.file_bytes = await self.file.read()
            return self.file_bytes
        elif self.file_base64_str is not None:
            self.file_bytes = base64.b64decode(self.file_base64_str)
            return self.file_bytes
        elif self.file_bytes is not None:
            return self.file_bytes
        else:
            return b""

    async def get_hash(self) -> str:
        """
        获取文件哈希值
        """
        file_bytes = await self.get_bytes()
        filename = self.save_name or self.file.filename
        return hashlib.md5(file_bytes + filename.encode()).hexdigest()

    async def check_limit(self):
        """
        检查文件是否符合上传限制
        """
        file_size = await self.get_size()
        file_mime_type = await self.get_mime_type()
        if self.limit_size and file_size > self.limit_size:
            raise ValueError("文件大小超出限制")
        if self.limit_type and not any(t in file_mime_type for t in self.limit_type):
            raise ValueError("文件类型不允许")
        if self.limit_image_width and self.limit_image_height:
            # 检查图片尺寸
            with Image.open(BytesIO(await self.get_bytes())) as img:
                width, height = img.size
                if width > self.limit_image_width or height > self.limit_image_height:
                    raise ValueError("图片尺寸超出限制")

    def path(self, path: str) -> "Storage":
        """
        设置保存路径
        """
        self.save_path = path
        return self

    def name(self, name: str) -> "Storage":
        """
        设置保存文件名
        """
        self.save_name = name
        return self

    async def save(self) -> FileInfo:
        """
        保存文件
        """
        try:
            # 根据驱动类型选择保存方式
            if self.driver == "local":
                return await self._save_local()
            elif self.driver == "oss" and self.oss_config:
                return await self._save_oss()
            elif self.driver == "minio" and self.minio_config:
                return await self._save_minio()
            else:
                # 默认本地保存
                return await self._save_local()
        except Exception as e:
            raise e

    async def _save_local(self) -> FileInfo:
        """
        本地保存文件
        """
        # 检查文件是否符合上传限制
        await self.check_limit()

        ext = await self.get_ext()
        # 处理文件名
        if self.rand_name:
            filename = f"{uuid.uuid4()}{ext}"
        else:
            filename = self.save_name or self.file.filename or f"{uuid.uuid4()}{ext}"

        # 构建完整路径
        if self.save_path:
            save_path = os.path.join(self.save_path, filename)
        else:
            save_path = filename

        # 确保目录存在
        directory = os.path.dirname(save_path)
        if directory and not os.path.exists(directory):
            os.makedirs(directory)

        file_bytes = await self.get_bytes()
        file_size = await self.get_size()
        file_mime_type = await self.get_mime_type()

        # 保存文件
        with open(save_path, "wb") as f:
            f.write(file_bytes)

        # 创建文件信息
        file_info = FileInfo(
            name=filename,
            ext=ext,
            path=save_path,
            size=file_size,
            mime_type=file_mime_type,
            url=f"{save_path}",
            hash=await self.get_hash(),
        )
        return file_info

    async def _save_oss(self) -> FileInfo:
        """
        OSS保存文件（模拟实现）
        """
        # 这里应该实现真实的OSS上传逻辑
        # 为简化示例，仍然保存到本地
        return await self._save_local()

    async def _save_minio(self) -> FileInfo:
        """
        Minio保存文件（模拟实现）
        """
        # 这里应该实现真实的Minio上传逻辑
        # 为简化示例，仍然保存到本地
        return await self._save_local()



import re

import inflection
from tortoise.exceptions import IntegrityError
from tortoise.queryset import QuerySet
from tortoise.transactions import in_transaction

from quark import Message, Request
from quark.loader import load_resource_classes
from quark.models import Permission
from quark.template.action import Action


# 辅助函数：PascalCase转换
def pascal_case(path: str, method: str) -> str:
    s = path.replace("/api/admin/", "").replace("/", "_") + "_" + method.lower()
    # 简单实现，将下划线分隔转成PascalCase
    parts = re.split(r"[^a-zA-Z0-9]", s)
    return "".join(word.capitalize() for word in parts if word)


def paser_routes(routes: list, class_type: str) -> list:
    new_permissions = []
    classes = load_resource_classes(class_type)
    for get_class in classes:
        for route in routes:
            path = route["path"].replace(
                "{resource}",
                inflection.camelize(get_class.__name__, False),
            )
            name = pascal_case(path, route["method"])
            new_permissions.append(
                Permission(
                    name=name,
                    method=route["method"],
                    path=path,
                    guard_name="admin",
                )
            )
    return new_permissions


class SyncPermission(Action):
    name = "同步权限"
    reload = "table"
    with_loading = True
    only_on_index = True
    action_type = "ajax"

    async def handle(self, request: Request, query: QuerySet):

        # 需要过滤对应路由
        routes = request.app.routes

        # 从数据库查询所有权限名
        existing_names = await Permission.all().values_list("name", flat=True)

        new_permissions = []
        names_set = set(existing_names)

        for route in routes:
            path = getattr(route, "path", "")
            methods = getattr(route, "methods", [])

            if not path.startswith("/api/admin"):
                continue

            if path not in [
                "/api/admin/dashboard/{resource}/index",
                "/api/admin/layout/{resource}/index",
                "/api/admin/login/{resource}/index",
                "/api/admin/login/{resource}/captchaId",
                "/api/admin/login/{resource}/captcha/{id}",
                "/api/admin/login/{resource}/handle",
                "/api/admin/logout/{resource}/handle",
                "/api/admin/{resource}/index",
                "/api/admin/{resource}/editable",
                "/api/admin/{resource}/create",
                "/api/admin/{resource}/store",
                "/api/admin/{resource}/edit",
                "/api/admin/{resource}/edit/values",
                "/api/admin/{resource}/save",
                "/api/admin/{resource}/import",
                "/api/admin/{resource}/export",
                "/api/admin/{resource}/detail",
                "/api/admin/{resource}/detail/values",
                "/api/admin/{resource}/import/template",
                "/api/admin/{resource}/form",
                "/api/admin/{resource}/action/{uriKey}",
                "/api/admin/{resource}/action/{uriKey}/values",
                "/api/admin/upload/{resource}/getList",
                "/api/admin/upload/{resource}/delete",
                "/api/admin/upload/{resource}/crop",
                "/api/admin/upload/{resource}/handle",
                "/api/admin/upload/{resource}/base64Handle",
            ]:
                for method in methods:
                    name = pascal_case(path, method)
                    if name not in names_set and all(
                        p.name != name for p in new_permissions
                    ):
                        new_permissions.append(
                            Permission(
                                name=name,
                                method=method,
                                path=path,
                                guard_name="admin",
                            )
                        )

        # 仪表盘
        classes = load_resource_classes("Dashboard")
        for get_class in classes:
            path = "/api/admin/dashboard/{resource}/index"
            method = "GET"
            path = path.replace(
                "{resource}",
                inflection.camelize(get_class.__name__.lower(), False),
            )
            name = pascal_case(path, method)
            if name not in names_set and all(p.name != name for p in new_permissions):
                new_permissions.append(
                    Permission(
                        name=name,
                        method=method,
                        path=path,
                        guard_name="admin",
                    )
                )

        # 布局
        classes = load_resource_classes("Layout")
        path = "/api/admin/layout/{resource}/index"
        method = "GET"
        for get_class in classes:
            path = path.replace(
                "{resource}",
                inflection.camelize(get_class.__name__.lower(), False),
            )
            name = pascal_case(path, method)
            if name not in names_set and all(p.name != name for p in new_permissions):
                new_permissions.append(
                    Permission(
                        name=name,
                        method=method,
                        path=path,
                        guard_name="admin",
                    )
                )

        # 登录
        login_routes = [
            {
                "path": "/api/admin/login/{resource}/index",
                "method": "GET",
            },
            {
                "path": "/api/admin/login/{resource}/captchaId",
                "method": "GET",
            },
            {
                "path": "/api/admin/login/{resource}/captcha/{id}",
                "method": "GET",
            },
            {
                "path": "/api/admin/login/{resource}/handle",
                "method": "POST",
            },
            {
                "path": "/api/admin/logout/{resource}/handle",
                "method": "GET",
            },
        ]

        new_permissions += paser_routes(login_routes, "Login")

        # 资源
        resource_routes = [
            {
                "path": "/api/admin/{resource}/index",
                "method": "GET",
            },
            {
                "path": "/api/admin/{resource}/editable",
                "method": "GET",
            },
            {
                "path": "/api/admin/{resource}/create",
                "method": "GET",
            },
            {
                "path": "/api/admin/{resource}/store",
                "method": "POST",
            },
            {
                "path": "/api/admin/{resource}/edit",
                "method": "GET",
            },
            {
                "path": "/api/admin/{resource}/edit/values",
                "method": "GET",
            },
            {
                "path": "/api/admin/{resource}/save",
                "method": "POST",
            },
            {
                "path": "/api/admin/{resource}/import",
                "method": "POST",
            },
            {
                "path": "/api/admin/{resource}/export",
                "method": "GET",
            },
            {
                "path": "/api/admin/{resource}/detail",
                "method": "GET",
            },
            {
                "path": "/api/admin/{resource}/detail/values",
                "method": "GET",
            },
            {
                "path": "/api/admin/{resource}/import/template",
                "method": "GET",
            },
            {
                "path": "/api/admin/{resource}/form",
                "method": "GET",
            },
            {
                "path": "/api/admin/{resource}/action/{uriKey}",
                "method": "GET",
            },
        ]
        new_permissions += paser_routes(resource_routes, "Resource")

        # 上传
        upload_routes = [
            {
                "path": "/api/admin/upload/{resource}/getList",
                "method": "GET",
            },
            {
                "path": "/api/admin/upload/{resource}/delete",
                "method": "GET",
            },
            {
                "path": "/api/admin/upload/{resource}/delete",
                "method": "POST",
            },
            {
                "path": "/api/admin/upload/{resource}/crop",
                "method": "POST",
            },
            {
                "path": "/api/admin/upload/{resource}/handle",
                "method": "POST",
            },
            {
                "path": "/api/admin/upload/{resource}/base64Handle",
                "method": "POST",
            },
        ]
        new_permissions += paser_routes(upload_routes, "Upload")

        # 资源行为
        action_routes = [
            {
                "path": "/api/admin/{resource}/action/{uriKey}",
                "method": "GET",
            },
            {
                "path": "/api/admin/{resource}/action/{uriKey}/values",
                "method": "GET",
            },
        ]
        classes = load_resource_classes("Resource")
        for get_class in classes:
            object = await get_class().init(request)
            actions = await object.actions(request)
            for action in actions:
                uri_key = action.get_uri_key(action)
                action_type = action.get_action_type()
                if action_type == "dropdown":
                    for dropdown_action in action.get_actions():
                        current_uri_key = action.get_uri_key(dropdown_action)
                        for action_route in action_routes:
                            path = action_route["path"]
                            method = action_route["method"]
                            path = path.replace(
                                "{uriKey}",
                                current_uri_key,
                            )
                            path = path.replace(
                                "{resource}",
                                inflection.camelize(get_class.__name__, False),
                            )
                            name = pascal_case(path, method)
                            if name not in names_set and all(
                                p.name != name for p in new_permissions
                            ):
                                new_permissions.append(
                                    Permission(
                                        name=name,
                                        method=method,
                                        path=path,
                                        guard_name="admin",
                                    )
                                )
                else:
                    for action_route in action_routes:
                        path = action_route["path"]
                        method = action_route["method"]
                        path = path.replace(
                            "{uriKey}",
                            uri_key,
                        )
                        path = path.replace(
                            "{resource}",
                            inflection.camelize(get_class.__name__, False),
                        )
                        name = pascal_case(path, method)
                        if name not in names_set and all(
                            p.name != name for p in new_permissions
                        ):
                            new_permissions.append(
                                Permission(
                                    name=name,
                                    method=method,
                                    path=path,
                                    guard_name="admin",
                                )
                            )

        if not new_permissions:
            return Message.error("无新增权限")

        try:
            async with in_transaction():
                await Permission.bulk_create(new_permissions)
        except IntegrityError as e:
            return Message.error(str(e))

        return Message.success("操作成功")



from typing import Any, Dict, Optional

from pydantic import BaseModel, field_validator


class LoginData(BaseModel):
    username: str
    password: str
    captcha: Dict[str, str]

    @field_validator("captcha")
    def captcha_must_have_id_and_value(cls, v):
        if "uuid" not in v or not v["uuid"]:
            raise ValueError("验证码ID不能为空")
        if "value" not in v or not v["value"]:
            raise ValueError("验证码不能为空")
        return v


class OSSConfig(BaseModel):
    """
    阿里云 OSS 配置模型
    """

    access_key_id: str
    access_key_secret: str
    endpoint: str
    bucket: str
    domain: Optional[str] = None
    is_https: bool = True
    path_prefix: Optional[str] = None
    acl: str = "default"


class MinioConfig(BaseModel):
    """
    MinIO 配置模型
    """

    endpoint: str
    access_key: str
    secret_key: str
    bucket: str
    secure: bool = True
    region: Optional[str] = None
    domain: Optional[str] = None
    path_prefix: Optional[str] = None


class FileInfo(BaseModel):
    """
    文件信息模型
    """

    name: str
    ext: str
    path: str
    size: int
    mime_type: str
    url: str
    hash: str
    extra: Optional[Dict[str, Any]] = None


class ImageListRequest(BaseModel):
    categoryId: Optional[int] = None
    name: Optional[str] = None
    createtime: Optional[str] = None
    page: int = 1


class ImageDeleteRequest(BaseModel):
    id: int


class ImageCropRequest(BaseModel):
    id: int
    file: str

class UserInfoResponse(BaseModel):
    id: int
    username: str
    nickname: str
    email: str
    phone: str
    avatar: str
    roles: list[str]
    buttons: list[str]


from pydantic import Field, model_validator
from typing import Any, List, Optional, Union
from ..component import Component


class FieldNames(Component):
    """
    定义树节点字段名称的结构。

    Attributes:
        title (str): 树节点的标题字段名称。
        key (str): 树节点的键字段名称。
        children (str): 树节点的子节点字段名称。
    """

    title: str
    key: str
    children: str


class TreeData(Component):
    """
    定义树节点的数据结构。

    Attributes:
        title (str): 树节点的标题。
        key (Any): 树节点的键，用于标识节点。
        children (List['TreeData']): 子节点列表。
        checkable (Optional[bool]): 是否可选中复选框。
        disable_checkbox (Optional[bool]): 是否禁用复选框。
        disabled (Optional[bool]): 是否禁用节点。
        icon (Optional[Any]): 自定义图标。
        is_leaf (Optional[bool]): 是否为叶子节点。
        selectable (Optional[bool]): 是否可选中节点。
    """

    title: Any
    key: Any
    children: List["TreeData"] = Field(default_factory=list)
    checkable: Optional[bool] = None
    disable_checkbox: Optional[bool] = None
    disabled: Optional[bool] = None
    icon: Optional[Any] = None
    is_leaf: Optional[bool] = None
    selectable: Optional[bool] = None


class TreeBar(Component):
    """
    定义树形组件的数据结构和操作方法。

    Attributes:
        component (str): 组件名称。
        name (str): 字段名，支持数组。
        auto_expand_parent (bool): 是否自动展开父节点。
        block_node (bool): 是否节点占据一行。
        checkable (bool): 是否节点前添加 Checkbox 复选框。
        checked_keys (List[Any]): 受控选中复选框的树节点。
        check_strictly (bool): checkable 状态下节点选择完全受控。
        default_checked_keys (List[Any]): 默认选中复选框的树节点。
        default_expand_all (bool): 默认展开所有树节点。
        default_expanded_keys (List[Any]): 默认展开指定的树节点。
        default_expand_parent (bool): 默认展开父节点。
        default_selected_keys (List[Any]): 默认选中的树节点。
        default_value (Optional[Any]): 默认选中的选项。
        disabled (bool): 整组失效。
        draggable (bool): 设置节点可拖拽。
        expanded_keys (List[Any]): 受控展开指定的树节点。
        field_names (Optional[FieldNames]): 自定义 options 中 label value children 的字段。
        height (Optional[int]): 设置虚拟滚动容器高度。
        icon (Optional[Any]): 自定义树节点图标。
        multiple (bool): 支持点选多个节点。
        placeholder (str): 占位文本。
        root_class_name (Optional[str]): 添加在 Tree 最外层的 className。
        root_style (Optional[Any]): 添加在 Tree 最外层的 style。
        selectable (bool): 是否可选中。
        selected_keys (List[Any]): 受控设置选中的树节点。
        show_icon (bool): 是否展示 TreeNode title 前的图标。
        show_line (bool): 是否展示连接线。
        switcher_icon (Optional[Any]): 自定义树节点的展开/折叠图标。
        tree_data (List[TreeData]): treeNodes 数据。
        value (Optional[Any]): 指定当前选中的条目。
        virtual (bool): 设置 false 时关闭虚拟滚动。
        style (dict): 自定义样式。
    """

    component: str = "treeBar"
    name: str = "treeBarSelectedKeys"
    auto_expand_parent: bool = False
    block_node: bool = False
    checkable: bool = False
    checked_keys: Optional[List[Any]] = None
    check_strictly: bool = False
    default_checked_keys: Optional[List[Any]] = None
    default_expand_all: bool = True
    default_expanded_keys: Optional[List[Any]] = None
    default_expand_parent: bool = False
    default_selected_keys: Optional[List[Any]] = None
    default_value: Optional[Any] = None
    disabled: bool = False
    draggable: bool = False
    expanded_keys: Optional[List[Any]]= None
    field_names: Optional[FieldNames] = None
    height: Optional[int] = None
    icon: Optional[Any] = None
    multiple: bool = False
    placeholder: str = "请输入搜索内容"
    root_class_name: Optional[str] = None
    root_style: Optional[Any] = None
    selectable: bool = True
    selected_keys: Optional[List[Any]] = None
    show_icon: bool = False
    show_line: bool = True
    switcher_icon: Optional[Any] = None
    tree_data: Optional[List[TreeData]] = None
    value: Optional[Any] = None
    virtual: bool = False
    style: dict = Field(default_factory=dict)

    @model_validator(mode="after")
    def init(self):
        self.set_key()
        return self

    # 设置方法（链式调用）
    def set_name(self, name: str):
        """
        设置字段名。

        Args:
            name (str): 字段名。

        Returns:
            TreeBar: 当前实例。
        """
        self.name = name
        return self

    def set_width(self, width: Union[str, int]):
        """
        设置宽度。

        Args:
            width (Union[str, int]): 宽度值。

        Returns:
            TreeBar: 当前实例。
        """
        self.style["width"] = width
        return self

    def set_auto_expand_parent(self, auto_expand_parent: bool):
        """
        设置是否自动展开父节点。

        Args:
            auto_expand_parent (bool): 是否自动展开父节点。

        Returns:
            TreeBar: 当前实例。
        """
        self.auto_expand_parent = auto_expand_parent
        return self

    def set_block_node(self, block_node: bool):
        """
        设置是否节点占据一行。

        Args:
            block_node (bool): 是否节点占据一行。

        Returns:
            TreeBar: 当前实例。
        """
        self.block_node = block_node
        return self

    def set_checkable(self, checkable: bool):
        """
        设置是否节点前添加 Checkbox 复选框。

        Args:
            checkable (bool): 是否节点前添加 Checkbox 复选框。

        Returns:
            TreeBar: 当前实例。
        """
        self.checkable = checkable
        return self

    def set_checked_keys(self, checked_keys: List[Any]):
        """
        设置受控选中复选框的树节点。

        Args:
            checked_keys (List[Any]): 受控选中复选框的树节点。

        Returns:
            TreeBar: 当前实例。
        """
        self.checked_keys = checked_keys
        return self

    def set_check_strictly(self, check_strictly: bool):
        """
        设置 checkable 状态下节点选择完全受控。

        Args:
            check_strictly (bool): 是否 checkable 状态下节点选择完全受控。

        Returns:
            TreeBar: 当前实例。
        """
        self.check_strictly = check_strictly
        return self

    def set_default_checked_keys(self, default_checked_keys: List[Any]):
        """
        设置默认选中复选框的树节点。

        Args:
            default_checked_keys (List[Any]): 默认选中复选框的树节点。

        Returns:
            TreeBar: 当前实例。
        """
        self.default_checked_keys = default_checked_keys
        return self

    def set_default_expand_all(self, default_expand_all: bool):
        """
        设置默认展开所有树节点。

        Args:
            default_expand_all (bool): 是否默认展开所有树节点。

        Returns:
            TreeBar: 当前实例。
        """
        self.default_expand_all = default_expand_all
        return self

    def set_default_expanded_keys(self, default_expanded_keys: List[Any]):
        """
        设置默认展开指定的树节点。

        Args:
            default_expanded_keys (List[Any]): 默认展开指定的树节点。

        Returns:
            TreeBar: 当前实例。
        """
        self.default_expanded_keys = default_expanded_keys
        return self

    def set_default_expand_parent(self, default_expand_parent: bool):
        """
        设置默认展开父节点。

        Args:
            default_expand_parent (bool): 是否默认展开父节点。

        Returns:
            TreeBar: 当前实例。
        """
        self.default_expand_parent = default_expand_parent
        return self

    def set_default_selected_keys(self, default_selected_keys: List[Any]):
        """
        设置默认选中的树节点。

        Args:
            default_selected_keys (List[Any]): 默认选中的树节点。

        Returns:
            TreeBar: 当前实例。
        """
        self.default_selected_keys = default_selected_keys
        return self

    def set_draggable(self, draggable: bool):
        """
        设置是否节点可拖拽。

        Args:
            draggable (bool): 是否节点可拖拽。

        Returns:
            TreeBar: 当前实例。
        """
        self.draggable = draggable
        return self

    def set_expanded_keys(self, expanded_keys: List[Any]):
        """
        设置受控展开指定的树节点。

        Args:
            expanded_keys (List[Any]): 受控展开指定的树节点。

        Returns:
            TreeBar: 当前实例。
        """
        self.expanded_keys = expanded_keys
        return self

    def set_field_names(self, field_names: FieldNames):
        """
        设置自定义 options 中 label value children 的字段。

        Args:
            field_names (FieldNames): 自定义字段名称。

        Returns:
            TreeBar: 当前实例。
        """
        self.field_names = field_names
        return self

    def set_height(self, height: int):
        """
        设置虚拟滚动容器高度。

        Args:
            height (int): 高度值。

        Returns:
            TreeBar: 当前实例。
        """
        self.height = height
        return self

    def set_icon(self, icon: Any):
        """
        设置自定义树节点图标。

        Args:
            icon (Any): 图标。

        Returns:
            TreeBar: 当前实例。
        """
        self.icon = icon
        return self

    def set_multiple(self, multiple: bool):
        """
        设置是否支持点选多个节点。

        Args:
            multiple (bool): 是否支持点选多个节点。

        Returns:
            TreeBar: 当前实例。
        """
        self.multiple = multiple
        return self

    def set_placeholder(self, placeholder: str):
        """
        设置占位文本。

        Args:
            placeholder (str): 占位文本。

        Returns:
            TreeBar: 当前实例。
        """
        self.placeholder = placeholder
        return self

    def set_root_class_name(self, root_class_name: str):
        """
        设置添加在 Tree 最外层的 className。

        Args:
            root_class_name (str): className。

        Returns:
            TreeBar: 当前实例。
        """
        self.root_class_name = root_class_name
        return self

    def set_root_style(self, root_style: Any):
        """
        设置添加在 Tree 最外层的 style。

        Args:
            root_style (Any): style。

        Returns:
            TreeBar: 当前实例。
        """
        self.root_style = root_style
        return self

    def set_selectable(self, selectable: bool):
        """
        设置是否可选中。

        Args:
            selectable (bool): 是否可选中。

        Returns:
            TreeBar: 当前实例。
        """
        self.selectable = selectable
        return self

    def set_selected_keys(self, selected_keys: List[Any]):
        """
        设置受控选中的树节点。

        Args:
            selected_keys (List[Any]): 受控选中的树节点。

        Returns:
            TreeBar: 当前实例。
        """
        self.selected_keys = selected_keys
        return self

    def set_show_icon(self, show_icon: bool):
        """
        设置是否展示 TreeNode title 前的图标。

        Args:
            show_icon (bool): 是否展示图标。

        Returns:
            TreeBar: 当前实例。
        """
        self.show_icon = show_icon
        return self

    def set_show_line(self, show_line: bool):
        """
        设置是否展示连接线。

        Args:
            show_line (bool): 是否展示连接线。

        Returns:
            TreeBar: 当前实例。
        """
        self.show_line = show_line
        return self

    def set_switcher_icon(self, switcher_icon: Any):
        """
        设置自定义树节点的展开/折叠图标。

        Args:
            switcher_icon (Any): 展开/折叠图标。

        Returns:
            TreeBar: 当前实例。
        """
        self.switcher_icon = switcher_icon
        return self

    def build_tree(
        self,
        items: Any,
        pid: Any,
        parent_key_name: str,
        key_name: str,
        title_name: str,
    ) -> List[TreeData]:
        """
        使用递归构建树结构。

        Args:
            items (Any): 树节点数据列表。
            pid (int): 父节点 ID。
            parent_key_name (str): 父节点键名称。
            key_name (str): 节点键名称。
            title_name (str): 节点标题名称。

        Returns:
            List[TreeData]: 树结构数据列表。
        """
        tree = []

        for item in items:
            # 支持对象和字典两种格式
            if hasattr(item, key_name):
                key = getattr(item, key_name)
                parent_key = getattr(item, parent_key_name)
                title = getattr(item, title_name)
            elif isinstance(item, dict):
                key = item.get(key_name)
                parent_key = item.get(parent_key_name)
                title = item.get(title_name)
            else:
                # 如果既不是对象也不是字典，跳过该项
                continue

            if parent_key == pid:
                children = self.build_tree(
                    items, key, parent_key_name, key_name, title_name
                )
                option = TreeData(title=title, key=key, children=children)
                tree.append(option)

        return tree

    def list_to_tree_data(
        self,
        list_data: Any,
        root_id: int,
        parent_key_name: str,
        key_name: str,
        title_name: str,
    ) -> List[TreeData]:
        """
        将列表数据转换为树结构数据。

        Args:
            list_data (Any): 列表数据。
            root_id (int): 根节点 ID。
            parent_key_name (str): 父节点键名称。
            key_name (str): 节点键名称。
            title_name (str): 节点标题名称。

        Returns:
            List[TreeData]: 树结构数据列表。
        """
        return self.build_tree(
            list_data, root_id, parent_key_name, key_name, title_name
        )

    def set_tree_data(self, *tree_data: Any):
        """
        设置树节点数据。

        Args:
            tree_data (Union[List[TreeData], int, str]): 树节点数据。

        Returns:
            TreeBar: 当前实例。
        """
        if len(tree_data) == 1 and isinstance(tree_data[0], list):
            self.tree_data = tree_data[0]
        elif len(tree_data) == 4:
            self.tree_data = self.list_to_tree_data(
                tree_data[0], 0, tree_data[1], tree_data[2], tree_data[3]
            )
        elif len(tree_data) == 5:
            self.tree_data = self.list_to_tree_data(
                tree_data[0], tree_data[1], tree_data[2], tree_data[3], tree_data[4]
            )
        return self

    def set_style(self, style: dict):
        """
        设置自定义样式。

        Args:
            style (dict): 样式字典。

        Returns:
            TreeBar: 当前实例。
        """
        self.style = style
        return self



from tortoise import fields
from tortoise.models import Model


class Config(Model):
    id = fields.IntField(pk=True)
    title = fields.CharField(max_length=255)
    type = fields.CharField(max_length=20)
    name = fields.CharField(max_length=255)
    sort = fields.IntField(default=0)
    group_name = fields.CharField(max_length=255)
    value = fields.CharField(max_length=2000, null=True)
    remark = fields.CharField(max_length=100, default="", null=False)
    status = fields.IntField(default=1)
    created_at = fields.DatetimeField(auto_now_add=True)
    updated_at = fields.DatetimeField(auto_now=True, null=True)

    class Meta(Model.Meta):
        table = "configs"

    @staticmethod
    async def seeder():
        seeders = [
            {
                "title": "网站名称",
                "type": "text",
                "name": "WEB_SITE_NAME",
                "group_name": "基本",
                "value": "QuarkCloud",
            },
            {
                "title": "关键字",
                "type": "text",
                "name": "WEB_SITE_KEYWORDS",
                "group_name": "基本",
                "value": "QuarkCloud",
            },
            {
                "title": "描述",
                "type": "textarea",
                "name": "WEB_SITE_DESCRIPTION",
                "group_name": "基本",
                "value": "QuarkCloud",
            },
            {
                "title": "Logo",
                "type": "picture",
                "name": "WEB_SITE_LOGO",
                "group_name": "基本",
                "value": "",
            },
            {
                "title": "统计代码",
                "type": "textarea",
                "name": "WEB_SITE_SCRIPT",
                "group_name": "基本",
                "value": "",
            },
            {
                "title": "网站域名",
                "type": "text",
                "name": "WEB_SITE_DOMAIN",
                "group_name": "基本",
                "value": "",
            },
            {
                "title": "网站版权",
                "type": "text",
                "name": "WEB_SITE_COPYRIGHT",
                "group_name": "基本",
                "value": "© Company 2018",
            },
            {
                "title": "开启SSL",
                "type": "switch",
                "name": "SSL_OPEN",
                "group_name": "基本",
                "value": "0",
            },
            {
                "title": "开启网站",
                "type": "switch",
                "name": "WEB_SITE_OPEN",
                "group_name": "基本",
                "value": "1",
            },
            {
                "title": "KeyID",
                "type": "text",
                "name": "OSS_ACCESS_KEY_ID",
                "group_name": "阿里云存储",
                "value": "",
                "remark": "你的AccessKeyID",
            },
            {
                "title": "KeySecret",
                "type": "text",
                "name": "OSS_ACCESS_KEY_SECRET",
                "group_name": "阿里云存储",
                "value": "",
                "remark": "你的AccessKeySecret",
            },
            {
                "title": "EndPoint",
                "type": "text",
                "name": "OSS_ENDPOINT",
                "group_name": "阿里云存储",
                "value": "",
                "remark": "地域节点",
            },
            {
                "title": "Bucket域名",
                "type": "text",
                "name": "OSS_BUCKET",
                "group_name": "阿里云存储",
                "value": "",
            },
            {
                "title": "自定义域名",
                "type": "text",
                "name": "OSS_MYDOMAIN",
                "group_name": "阿里云存储",
                "value": "",
                "remark": "例如：oss.web.com",
            },
            {
                "title": "开启云存储",
                "type": "switch",
                "name": "OSS_OPEN",
                "group_name": "阿里云存储",
                "value": "0",
            },
        ]

        for data in seeders:
            exists = await Config.filter(name=data["name"]).exists()
            if not exists:
                await Config.create(
                    title=data["title"],
                    type=data["type"],
                    name=data["name"],
                    group_name=data["group_name"],
                    value=data["value"],
                    remark=data.get("remark", ""),
                )



from typing import Dict, Any, Optional
from pydantic import model_validator
from .base import Base


class Editor(Base):

    component: str = "editorField"
    """
    组件名称
    """

    default_value: Optional[Any] = None
    """
    默认选中的选项
    """

    disabled: bool = False
    """
    整组失效
    """

    style: Optional[Dict[str, Any]] = None
    """
    自定义样式
    """

    value: Optional[Any] = None
    """
    指定选中项
    """

    @model_validator(mode="after")
    def init(self):
        self.style = {"height": 500, "width": "100%"}
        self.set_key()
        return self

    def set_width(self, width: Any):
        """
        设置组件宽度。

        Args:
            width (Any): 宽度值。

        Returns:
            Component: 当前组件实例，支持链式调用。
        """
        style = self.style.copy() if self.style else {}
        style["width"] = width
        self.style = style
        return self

    def set_height(self, height: Any):
        """
        设置组件高度。

        Args:
            height (Any): 高度值。

        Returns:
            Component: 当前组件实例，支持链式调用。
        """
        style = self.style.copy() if self.style else {}
        style["height"] = height
        self.style = style
        return self

    def set_default_value(self, default_value: Any):
        self.default_value = default_value
        return self

    def set_value(self, value: Any):
        self.value = value
        return self



from quark import Request
from quark.services.auth import AuthService
from quark.template.action import Link


class BatchExport(Link):

    def __init__(self, name: str = "批量导出"):
        self.name = name
        self.type = "default"
        self.size = "small"
        self.reload = "table"
        self.target = "_blank"
        self.batch = True
        self.with_confirm("确定要导出数据吗？", "导出数据可能会等待时间较长！", "modal")
        self.set_only_on_index(True)

    async def get_href(self, request: Request) -> str:
        path = request.url.path.replace("/index", "")
        token = AuthService(request).get_token()
        return f"{path}/export?id=${{id}}&token={token}"



from pydantic import model_validator
from typing import Any
from ..component import Component
from .item import Item
from .sub_menu import SubMenu
from .item_group import ItemGroup
from .divider import Divider


class Menu(Component):
    component: str = "menu"
    default_open_keys: Any = None
    default_selected_keys: Any = None
    inline_collapsed: bool = False
    inline_indent: int = 24
    mode: str = "vertical"
    multiple: bool = False
    selectable: bool = True
    sub_menu_close_delay: float = 0.1
    sub_menu_open_delay: float = 0
    theme: str = "light"
    trigger_sub_menu_action: str = "hover"
    items: Any = None

    @model_validator(mode="after")
    def init(self):
        self.set_key("DEFAULT_KEY", False)
        return self

    # 初始展开的 SubMenu 菜单项 key 数组
    def set_default_open_keys(self, default_open_keys: Any) -> "Component":
        self.default_open_keys = default_open_keys
        return self

    # 初始选中的菜单项 key 数组
    def set_default_selected_keys(self, default_selected_keys: Any) -> "Component":
        self.default_selected_keys = default_selected_keys
        return self

    # inline 时菜单是否收起状态
    def set_inline_collapsed(self, inline_collapsed: bool) -> "Component":
        self.inline_collapsed = inline_collapsed
        return self

    # inline 模式的菜单缩进宽度
    def set_inline_indent(self, inline_indent: int) -> "Component":
        self.inline_indent = inline_indent
        return self

    # 菜单类型，现在支持垂直、水平、和内嵌模式三种,vertical | horizontal | inline
    def set_mode(self, mode: str) -> "Component":
        self.mode = mode
        return self

    # 是否允许多选
    def set_multiple(self, multiple: bool) -> "Component":
        self.multiple = multiple
        return self

    # 是否允许选中
    def set_selectable(self, selectable: bool) -> "Component":
        self.selectable = selectable
        return self

    # 用户鼠标离开子菜单后关闭延时，单位：秒
    def set_sub_menu_close_delay(self, sub_menu_close_delay: float) -> "Component":
        self.sub_menu_close_delay = sub_menu_close_delay
        return self

    # 主题颜色,light | dark
    def set_theme(self, theme: str) -> "Component":
        self.theme = theme
        return self

    # SubMenu 展开/关闭的触发行为,hover | click
    def set_trigger_sub_menu_action(self, trigger_sub_menu_action: str) -> "Component":
        self.trigger_sub_menu_action = trigger_sub_menu_action
        return self

    # 设置菜单项
    def set_items(self, items: Any) -> "Component":
        self.items = items
        return self

    # 菜单分隔符
    def divider(self) -> Divider:
        return Divider()

    # 菜单分组
    def item_group(self, title: str, items: Any) -> ItemGroup:
        return ItemGroup().set_title(title).set_items(items)

    # 菜单项
    def item(self, label: str, title: str) -> Item:
        return Item().set_label(label).set_title(title)

    # 子菜单
    def sub_menu(self, title: str, items: Any) -> SubMenu:
        return SubMenu().set_title(title).set_items(items)



from typing import Any, List

from quark import Request, Resource, models
from quark.app import actions, searches
from quark.component.form import field
from quark.component.form.rule import Rule


class Config(Resource):
    """
    配置管理
    """

    async def init(self, request: Request):

        # 页面标题
        self.title = "配置"

        # 模型
        self.model = models.Config

        return self

    async def fields(self, request: Request) -> List[Any]:
        """字段定义"""
        return [
            field.id("id", "ID"),
            field.text("title", "标题").set_rules([Rule.required("标题必须填写")]),
            field.text("name", "名称")
            .set_rules([Rule.required("名称必须填写")])
            .set_creation_rules(
                [
                    Rule.unique("configs", "name", "名称已存在"),
                ]
            )
            .set_update_rules(
                [
                    Rule.unique("configs", "name", "{id}", "名称已存在"),
                ]
            ),
            field.radio("type", "类型")
            .set_options(
                [
                    field.radio_option("文本", "text"),
                    field.radio_option("文本域", "textarea"),
                    field.radio_option("图片", "picture"),
                    field.radio_option("文件", "file"),
                    field.radio_option("开关", "switch"),
                ]
            )
            .set_default_value("text")
            .only_on_forms(),
            field.number("sort", "排序").set_default_value(0).only_on_forms(),
            field.text("group_name", "分组名称")
            .set_rules([Rule.required("分组名称必须填写")])
            .only_on_forms(),
            field.textarea("remark", "备注"),
            field.switch("status", "状态")
            .set_editable(True)
            .set_true_value("正常")
            .set_false_value("禁用")
            .set_default_value(True),
        ]

    async def searches(self, request: Request) -> List[Any]:
        """搜索项定义"""
        return [
            searches.Input("title", "标题"),
            searches.Input("name", "名称"),
            searches.Status(),
        ]

    async def actions(self, request: Request) -> List[Any]:
        """行为定义"""
        return [
            actions.CreateDrawer(self),
            actions.BatchDelete(),
            actions.EditDrawer(self),
            actions.Delete(),
        ]



from typing import Any, Dict, List

from quark import Request

from .action import Action


class DrawerForm(Action):
    """
    表示一个抽屉式表单组件，支持配置宽度、按钮文案、API 类型等。
    """

    action_type: str = "drawerForm"

    # 抽屉弹出层宽度，默认值为 520
    width: int = 520

    # 关闭时销毁弹出层里的子元素，默认为 False
    destroy_on_close: bool = False

    # 设置按钮类型，primary | ghost | dashed | link | text | default
    type: str = "default"

    # 设置按钮大小,large | middle | small | default
    size: str = "small"

    # 获取取消按钮文案，默认为“取消”
    cancel_text: str = "取消"

    # 获取提交按钮文案，默认为“提交”
    submit_text: str = "提交"

    # 表单提交接口的类型，GET 或 POST，默认为 POST
    api_type: str = "POST"

    # 提交表单的数据是否打开新页面，只有在 GET 类型的时候有效，默认为 False
    target_blank: bool = False

    # 抽屉关闭时是否刷新表格数据，默认为 False
    reload: Any = "table"

    async def fields(self, request: Request) -> List[Any]:
        """
        表单字段定义
        """
        return []

    async def data(self, request: Request) -> Dict[str, Any]:
        """
        异步获取表单数据
        """
        return {}

    def get_width(self) -> int:
        """获取抽屉弹出层宽度"""
        return self.width

    def get_destroy_on_close(self) -> bool:
        """关闭时销毁 Modal 里的子元素"""
        return self.destroy_on_close

    def get_cancel_text(self) -> str:
        """获取取消按钮文案"""
        return self.cancel_text

    def get_submit_text(self) -> str:
        """获取提交按钮文案"""
        return self.submit_text

    def get_api_type(self) -> str:
        """表单提交接口的类型，GET 或 POST，默认为 POST"""
        return self.api_type

    def get_target_blank(self) -> bool:
        """提交表单的数据是否打开新页面，只有在 GET 类型的时候有效"""
        return self.target_blank



from typing import Any, Optional

from pydantic import BaseModel

from ...component.descriptions.descriptions import Descriptions as DescriptionsComponent


class Descriptions(BaseModel):
    title: Optional[str] = None
    col: Optional[int] = None

    def result(self, value: Any) -> DescriptionsComponent:
        return DescriptionsComponent().set_title(self.title).set_items(value)



from .models.config import Config
from .models.department import Department
from .models.menu import Menu
from .models.position import Position
from .models.role import Role
from .models.user import User


# 执行安装操作
async def setup_all():

    # 初始化数据
    await User.seeder()
    await Role.seeder()
    await Position.seeder()
    await Menu.seeder()
    await Department.seeder()
    await Config.seeder()



from typing import Any, Optional, Union

from pydantic import Field, model_validator

from .base import Base


class Password(Base):
    component: str = "passwordField"
    addon_after: Optional[Any] = None
    addon_before: Optional[Any] = None
    allow_clear: bool = False
    bordered: bool = True
    default_value: Optional[Any] = None
    disabled: Optional[Union[bool, str]] = None
    id: str = None
    max_length: int = 200
    show_count: bool = False
    status: str = None
    prefix: Optional[Any] = None
    size: str = None
    suffix: Optional[Any] = None
    type: str = None
    value: Optional[Any] = None
    placeholder: str = "请输入"
    style: dict = Field(default_factory=dict)
    visibility_toggle: bool = None

    @model_validator(mode="after")
    def init(self):
        self.set_key()
        self.set_width("200px")
        return self

    def set_addon_after(self, addon_after: Any):
        self.addon_after = addon_after
        return self

    def set_addon_before(self, addon_before: Any):
        self.addon_before = addon_before
        return self

    def set_allow_clear(self, allow_clear: bool):
        self.allow_clear = allow_clear
        return self

    def set_bordered(self, bordered: bool):
        self.bordered = bordered
        return self

    def set_default_value(self, default_value: Any):
        self.default_value = default_value
        return self

    def set_value(self, value: Any):
        self.value = value
        return self

    def set_id(self, id: str):
        self.id = id
        return self

    def set_max_length(self, max_length: int):
        self.max_length = max_length
        return self

    def set_show_count(self, show_count: bool):
        self.show_count = show_count
        return self

    def set_status(self, status: str):
        self.status = status
        return self

    def set_placeholder(self, placeholder: str):
        self.placeholder = placeholder
        return self

    def set_prefix(self, prefix: Any):
        self.prefix = prefix
        return self

    def set_size(self, size: str):
        self.size = size
        return self

    def set_suffix(self, suffix: Any):
        self.suffix = suffix
        return self

    def set_type(self, type: str):
        self.type = type
        return self

    def set_visibility_toggle(self, visibility_toggle: bool):
        self.visibility_toggle = visibility_toggle
        return self



import json
from datetime import datetime
from typing import Any

from fastapi import Request

from quark import Message, Storage, Upload
from quark.models import Attachment
from quark.schemas import FileInfo, ImageCropRequest, ImageDeleteRequest
from quark.services import AttachmentCategoryService, AttachmentService, AuthService


class Image(Upload):

    async def init(self, request: Request):
        """
        初始化
        """
        # 限制文件大小
        self.limit_size = 1024 * 1024 * 1024 * 2

        # 限制文件类型
        self.limit_type = [
            "image/png",
            "image/gif",
            "image/jpeg",
            "image/svg+xml",
        ]

        # 设置文件上传路径
        self.save_path = (
            f"./web/app/storage/images/{datetime.now().strftime('%Y%m%d')}/"
        )

        return self

    async def get_list(self, request: Request):
        """
        获取文件列表
        """
        try:
            # 从request中获取参数
            category_id = request.query_params.get("categoryId")
            name = request.query_params.get("name")
            createtime = request.query_params.get("createtime")
            page = request.query_params.get("page", "1")

            # 类型转换
            try:
                category_id = int(category_id) if category_id else None
                page = int(page) if page else 1
            except ValueError:
                page = 1

            current_admin = await AuthService(request).get_current_admin()

            # 使用服务层获取数据
            images, total = await AttachmentService().get_list_by_search(
                current_admin.id, "IMAGE", category_id, name, createtime, page
            )

            # 获取分类列表
            categorys = await AttachmentCategoryService().get_list(current_admin.id)
        except Exception as e:
            return Message.error(str(e))

        return Message.success(
            "获取成功",
            {
                "pagination": {
                    "current": page,
                    "defaultCurrent": 1,
                    "pageSize": 8,
                    "total": total,
                },
                "list": images,
                "categorys": categorys,
            },
        )

    async def delete(self, request: Request):
        """
        图片删除
        """
        try:
            # 从request中解析JSON数据
            request_data = await request.json()

            image_delete_req = ImageDeleteRequest(**request_data)

            await AttachmentService().delete_by_id(image_delete_req.id)

        except Exception as e:
            return Message.error(str(e))

        return Message.success("操作成功")

    async def crop(self, request: Request):
        """
        图片裁剪
        """
        # 从request中解析JSON数据
        request_data = await request.json()

        image_crop_req = ImageCropRequest(**request_data)

        # 获取查询参数
        limit_w = request.query_params.get("limitW")
        limit_h = request.query_params.get("limitH")

        # 获取图片信息
        image_info = await AttachmentService().get_info_by_id(image_crop_req.id)
        if not image_info:
            return Message.error("图片不存在")

        # 解析base64数据
        files = image_crop_req.file.split(",")
        if len(files) != 2:
            return Message.error("格式错误")

        # 获取尺寸限制
        limit_image_width = getattr(self, "limit_image_width", 0)
        limit_image_height = getattr(self, "limit_image_height", 0)

        if limit_w:
            try:
                limit_image_width = int(limit_w)
            except ValueError:
                pass

        if limit_h:
            try:
                limit_image_height = int(limit_h)
            except ValueError:
                pass

        # 创建存储实例
        storage = Storage(
            limit_size=self.limit_size,
            limit_type=self.limit_type,
            limit_image_width=limit_image_width,
            limit_image_height=limit_image_height,
            driver=self.driver,
            save_path=self.save_path,
            file_base64_str=files[1],
        )

        # 处理文件
        file_paths = image_info.path.split("/")
        file_name = file_paths[-1]

        result = await storage.name(file_name).save()

        # 重写URL
        if self.driver == "local":
            result.url = await AttachmentService().get_image_url(result.url)

        # 更新数据库
        extra = ""
        if result.extra:
            extra = json.dumps(result.extra)

        current_admin = await AuthService(request).get_current_admin()

        attachment = Attachment(
            source="ADMIN",
            uid=current_admin.id,
            name=result.name,
            type="IMAGE",
            size=result.size,
            ext=result.ext,
            path=result.path,
            url=result.url,
            hash=result.hash,
            extra=extra,
            status=1,
        )

        await AttachmentService().update_by_id(
            image_info.id,attachment
        )

        # 重新获取更新后的数据
        updated_attachment = await AttachmentService().get_info_by_id(image_info.id)

        return Message.success(
            "操作成功",
            {
                "name": updated_attachment.name,
                "size": updated_attachment.size,
                "ext": updated_attachment.ext,
                "path": updated_attachment.path,
                "url": updated_attachment.url,
                "hash": updated_attachment.hash,
                "extra": (
                    json.loads(updated_attachment.extra)
                    if updated_attachment.extra
                    else None
                ),
            },
        )

    async def before_handle(self, request: Request, storage: Storage) -> Any:
        """
        上传前回调
        """
        try:
            file_hash = await storage.get_hash()
            image_info = await AttachmentService().get_info_by_hash(file_hash)

            if image_info and image_info.id != 0:
                extra = {}
                if image_info.extra:
                    try:
                        extra = json.loads(image_info.extra)
                    except:
                        pass

                file_info = {
                    "name": image_info.name,
                    "size": image_info.size,
                    "ext": image_info.ext,
                    "path": image_info.path,
                    "url": image_info.url,
                    "hash": image_info.hash,
                    "extra": extra,
                }

                return Message.success("获取成功", file_info)

            return None
        except Exception as e:
            return Message.error(str(e))

    async def after_handle(self, request: Request, result: FileInfo) -> Any:
        """
        上传完成后回调
        """
        try:
            # 重写url
            if self.driver == "local":
                result.url = await AttachmentService().get_image_url(result.url)

            extra = ""
            if result.extra:
                extra = json.dumps(result.extra)

            current_admin = await AuthService(request).get_current_admin()

            attachment = Attachment(
                source="ADMIN",
                uid=current_admin.id,
                name=result.name,
                type="IMAGE",
                size=result.size,
                ext=result.ext,
                path=result.path,
                url=result.url,
                hash=result.hash,
                extra=extra,
                status=1,
            )

            # 插入数据库
            attachment_id = await AttachmentService().insert_get_id(attachment)
            return Message.success(
                "上传成功",
                {
                    "id": attachment_id,
                    "contentType": result.mime_type,
                    "ext": result.ext,
                    "hash": result.hash,
                    "name": result.name,
                    "path": result.path,
                    "size": result.size,
                    "url": result.url,
                    "extra": result.extra,
                },
            )

        except Exception as e:
            return Message.error(str(e))


class File(Upload):
    """
    文件上传处理类
    """

    async def init(self, request: Request):
        """
        初始化文件上传配置

        Args:
            ctx: Quark上下文对象

        Returns:
            File: 配置后的File对象
        """
        # 限制文件大小为2GB
        self.limit_size = 1024 * 1024 * 1024 * 2

        # 限制文件类型
        self.limit_type = [
            "image/png",
            "image/gif",
            "image/jpeg",
            "video/mp4",
            "video/mpeg",
            "application/x-xls",
            "application/x-ppt",
            "application/msword",
            "application/zip",
            "application/pdf",
            "application/vnd.ms-excel",
            "application/vnd.ms-powerpoint",
            "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet",
            "application/vnd.openxmlformats-officedocument.wordprocessingml.document",
            "application/vnd.openxmlformats-officedocument.presentationml.presentation",
        ]

        # 设置文件上传路径
        self.save_path = f"./web/app/storage/files/{datetime.now().strftime('%Y%m%d')}/"

        return self

    async def before_handle(self, request: Request, storage: Storage) -> Any:
        """
        上传前回调处理

        Args:
            request: 请求对象
            storage: 存储对象

        Returns:
            Any: 处理结果
        """
        try:
            file_hash = await storage.get_hash()
        except Exception as e:
            return Message.error(str(e))

        try:
            file_info = await AttachmentService().get_info_by_hash(file_hash)
        except Exception as e:
            return Message.error(str(e))

        if file_info and file_info.id != 0:
            extra = {}
            if file_info.extra:
                try:
                    extra = json.loads(file_info.extra)
                except (json.JSONDecodeError, TypeError):
                    pass

            file_info_obj = FileInfo(
                name=file_info.name,
                size=file_info.size,
                ext=file_info.ext,
                path=file_info.path,
                url=file_info.url,
                hash=file_info.hash,
                extra=extra,
            )
            return Message.success("获取成功", file_info_obj)

        return None

    async def after_handle(self, request: Request, result: FileInfo) -> Any:
        """
        上传完成后回调处理

        Args:
            request: 请求对象
            result: 文件信息对象

        Returns:
            Any: 处理结果
        """

        # 重写url
        if self.driver == "local":
            result.url = await AttachmentService().get_file_url(result.url)

        try:
            admin_info = await AuthService(request).get_current_admin()
        except Exception as e:
            return Message.error(str(e))

        extra = ""
        if result.extra is not None:
            try:
                extra_data = json.dumps(result.extra)
                extra = extra_data
            except (TypeError, ValueError):
                pass

        # 插入数据库
        try:
            attachment = Attachment(
                source="ADMIN",
                uid=admin_info.id,
                name=result.name,
                type="FILE",
                size=result.size,
                ext=result.ext,
                path=result.path,
                url=result.url,
                hash=result.hash,
                extra=extra,
                status=1,
            )
            id = await AttachmentService().insert_get_id(attachment)
        except Exception as e:
            return Message.error(str(e))

        return Message.success(
            "上传成功",
            {
                "id": id,
                "contentType": result.mime_type,
                "ext": result.ext,
                "hash": result.hash,
                "name": result.name,
                "path": result.path,
                "size": result.size,
                "url": result.url,
                "extra": result.extra,
            },
        )



import base64
import random
import string
import uuid
from io import BytesIO
from typing import Any, Optional

from captcha.image import ImageCaptcha
from pydantic import BaseModel, Field

from quark import Message, Request
from quark.services.auth import AuthService
from quark.services.menu import MenuService

from .. import cache
from ..component.auth.auth import Auth as AuthComponent
from ..component.divider.divider import Divider
from ..component.tabs.tabs import Tabs


class Auth(BaseModel):
    """登录组件"""

    # 登录接口
    login_api: str = Field(default="/api/admin/auth/index/index")

    # 用户信息接口
    user_info_api: str = Field(default="/api/admin/auth/index/userInfo")

    # 用户路由接口
    user_routes_api: str = Field(default="/api/admin/auth/index/userRoutes")

    # 登录后跳转地址
    redirect: str = Field(default="/layout/index?api=/api/admin/dashboard/index/index")

    # Logo
    logo: Optional[Any] = Field(default=None)

    # 标题
    title: str = Field(default="QuarkPy")

    async def init(self, request: Request):
        """初始化"""
        return self

    async def captcha(self, request: Request):
        id = str(uuid.uuid4())
        captcha_text = "".join(random.choices(string.digits, k=4))
        await cache.set(id, captcha_text, 60)

        image = ImageCaptcha(width=170, height=50)
        captcha_image = image.generate_image(captcha_text)

        buffer = BytesIO()
        captcha_image.save(buffer, format="PNG")
        buffer.seek(0)

        return Message.success(
            "获取成功",
            {
                "captchaEnabled": True,
                "img": base64.b64encode(buffer.getvalue()).decode("utf-8"),
                "uuid": id,
            },
        )

    async def fields(self, request: Request):
        return []

    async def login(self, request: Request):
        return Message.error("请实现登录方法")

    async def user_info(self, request: Request):
        user_info = await AuthService(request).get_current_admin()
        return Message.success("ok",user_info)

    async def user_routes(self, request: Request):
        user_info = await AuthService(request).get_current_user("admin")
        menus = await MenuService().get_list_by_user_id(user_info.id)
        return Message.success("ok", {
            "routes": menus,
            "home": "home"
        })


    async def logout(self, request: Request):
        return Message.success("退出成功")

    async def fields_within_components(self, request: Request):

        # 获取字段
        fields = await self.fields(request)

        # 解析创建页表单组件内的字段
        items = await self.form_fields_parser(fields, request)

        return items

    async def form_fields_parser(self, fields: Any, request: Request):
        items = []
        path = request.url.path
        # 解析字段
        if isinstance(fields, list):
            for v in fields:
                if hasattr(v, "body"):
                    # 获取内容值
                    body = v.body

                    # 解析值
                    get_fields = await self.form_fields_parser(body, request)

                    # 更新值
                    v.body = get_fields

                    items.append(v)
                else:
                    component = getattr(v, "component", "")
                    if "Field" in component:
                        # 判断是否在创建页面
                        if (
                            hasattr(v, "is_shown_on_creation")
                            and v.is_shown_on_creation()
                        ):
                            # 生成前端验证规则
                            if hasattr(v, "build_frontend_rules"):
                                v.build_frontend_rules(path)

                            # 组合数据
                            items.append(v)
                    else:
                        items.append(v)

        return items

    async def render(self, request: Request):

        # 登录接口
        login_api = self.login_api

        # 用户信息接口
        user_info_api = self.user_info_api

        # 用户路由接口
        user_routes_api = self.user_routes_api

        # 登录后跳转地址
        redirect = self.redirect

        # Logo
        logo = self.logo

        # 标题
        title = self.title

        # 包裹在组件内的字段
        fields = await self.fields_within_components(request)

        # 解析tabPane组件
        if isinstance(fields, list) and len(fields) > 0:
            component_name = getattr(fields[0], "component", "")

            if component_name == "tabPane":
                tab_component = Tabs().set_tab_panes(fields).set_centered(True)

                # 组件
                component = (
                    AuthComponent()
                    .set_login_api(login_api)
                    .set_user_info_api(user_info_api)
                    .set_user_routes_api(user_routes_api)
                    .set_redirect(redirect)
                    .set_logo(logo)
                    .set_title(title)
                    .set_body(tab_component)
                )
            else:
                fields = [Divider().set_style({"marginTop": "-15px"})] + fields

                # 组件
                component = (
                    AuthComponent()
                    .set_login_api(login_api)
                    .set_user_info_api(user_info_api)
                    .set_user_routes_api(user_routes_api)
                    .set_redirect(redirect)
                    .set_logo(logo)
                    .set_title(title)
                    .set_body(fields)
                )
        else:
            fields = [Divider().set_style({"marginTop": "-15px"})] + fields

            # 组件
            component = (
                AuthComponent()
                .set_login_api(login_api)
                .set_user_info_api(user_info_api)
                .set_user_routes_api(user_routes_api)
                .set_redirect(redirect)
                .set_logo(logo)
                .set_title(title)
                .set_body(fields)
            )

        return Message.success("退出成功", component)



from quark.template.action import Action


class FormStep(Action):

    def __init__(self):

        # 文字
        self.name = ["上一步", "下一步"]

        # 类型
        self.type = "default"

        # 行为类型
        self.action_type = "step"

        # 设置在表单页展示
        self.set_show_on_form()



from typing import Any

from tortoise.models import Model

from quark import Request

from .search import Search


class Input(Search):
    """输入框组件"""

    component = "inputField"

    def __init__(self, column: str = "", name: str = ""):
        self.column = column
        self.name = name

    def apply(self, request: Request, query: Model, value: Any) -> Model:
        """执行查询逻辑，子类可重写此方法"""
        return query.filter(**{f"{self.column}__icontains": value})



import json
from typing import Any, List

from tortoise.queryset import QuerySet

from quark import Request, Resource, models
from quark.app import actions, searches
from quark.component.form import field
from quark.services.attachment import AttachmentService


class Image(Resource):
    """
    图片管理
    """

    async def init(self, request: Request):

        self.title = "图片"

        # 模型
        self.model = models.Attachment

        return self

    async def index_query(self, request: Request) -> QuerySet:
        """
        列表查询
        """
        query = await self.query(request)
        return query.filter(type="IMAGE")

    async def fields(self, request: Request) -> List[Any]:
        """字段定义"""
        return [
            field.id("id", "ID"),
            field.text(
                "path",
                "显示",
                lambda row: f"<img src='{AttachmentService().get_image_url(row['id'])}' width=50 height=50 />",
            ),
            field.text("name", "名称").set_ellipsis(True),
            field.text("size", "大小").set_sorter(True),
            field.text(
                "extra", "尺寸", lambda row: self._parse_extra(row.get("extra"))
            ),
            field.text("ext", "扩展名"),
            field.datetime("created_at", "上传时间"),
        ]

    def _parse_extra(self, extra: Any) -> str:
        """解析尺寸字段"""
        if not extra:
            return ""
        try:
            data = json.loads(extra)
            if "width" in data and "height" in data:
                return f"{int(data['width'])}*{int(data['height'])}"
        except Exception:
            pass
        return ""

    async def searches(self, request: Request) -> List[Any]:
        """搜索项定义"""
        return [
            searches.Input("name", "名称"),
            searches.DatetimeRange("created_at", "上传时间"),
        ]

    async def actions(self, request: Request) -> List[Any]:
        """行为定义"""
        return [
            actions.BatchDelete(),
            actions.Delete(),
        ]



from typing import Any, List

from tortoise.queryset import QuerySet

from quark import Request, Resource, models
from quark.app import actions, searches
from quark.component.form import field


class File(Resource):
    """
    文件管理
    """

    async def init(self, request: Request):

        # 页面标题
        self.title = "文件"

        # 模型
        self.model = models.Attachment

        return self

    async def index_query(self, request: Request) -> QuerySet:
        """
        列表查询
        """
        query = await self.query(request)
        return query.filter(type="FILE")

    async def fields(self, request: Request) -> List[Any]:
        """字段定义"""
        return [
            field.id("id", "ID"),
            field.text("name", "名称"),
            field.text("size", "大小").set_sorter(True),
            field.text("ext", "扩展名"),
            field.datetime("created_at", "上传时间"),
        ]

    async def searches(self, request: Request) -> List[Any]:
        """搜索项定义"""
        return [
            searches.Input("name", "名称"),
            searches.DatetimeRange("created_at", "上传时间"),
        ]

    async def actions(self, request: Request) -> List[Any]:
        """行为定义"""
        return [
            actions.BatchDelete(),
            actions.Delete(),
        ]



from pydantic import Field, model_validator
from typing import Any, Dict, List, Optional
from ..component import Component

class PageContainer(Component):
    component: str = Field(default="pageContainer")
    content: Any = Field(None, description="内容区")
    extra_content: Optional[Any] = Field(None, description="额外内容区，位于 content 的右侧")
    tab_list: Optional[List[Dict[str, str]]] = Field(None, description="tab 标题列表")
    tab_active_key: Optional[str] = Field(None, description="当前高亮的 tab 项")
    tab_bar_extra_content: Optional[Any] = Field(None, description="tab bar 上额外的元素")
    header: Optional[Any] = Field(None, description="PageHeader 的所有属性")
    ghost: bool = Field(False, description="配置头部区域的背景颜色为透明")
    fixed_header: bool = Field(False, description="固定 pageHeader 的内容到顶部")
    affix_props: Optional[Any] = Field(None, description="固钉的配置，与 antd 完全相同")
    footer: Optional[Any] = Field(None, description="悬浮在底部的操作栏，传入一个数组，会自动加空格")
    body: Optional[Any] = Field(None, description="容器控件里面的内容")
    water_mark_props: Optional[Any] = Field(None, description="配置水印，Layout 会透传给 PageContainer，但是以 PageContainer 的配置优先")
    tab_props: Optional[Any] = Field(None, description="tab 的配置")
    style: Optional[Dict[str, Any]] = Field(None, description="样式")

    @model_validator(mode="after")
    def init(self):
        self.set_key("pageContainer")
        return self

    def set_style(self, style: Dict[str, Any]):
        # 设置样式
        self.style = style
        return self

    def set_content(self, content: Any):
        # 设置内容区
        self.content = content
        return self

    def set_extra_content(self, extra_content: Any):
        # 设置额外内容区
        self.extra_content = extra_content
        return self

    def set_tab_list(self, tab_list: List[Dict[str, str]]):
        # 设置 tab 标题列表
        self.tab_list = tab_list
        return self

    def set_tab_active_key(self, tab_active_key: str):
        # 设置当前高亮的 tab 项
        self.tab_active_key = tab_active_key
        return self

    def set_tab_bar_extra_content(self, tab_bar_extra_content: Any):
        # 设置 tab bar 上额外的元素
        self.tab_bar_extra_content = tab_bar_extra_content
        return self

    def set_header(self, header: Any):
        # 设置 PageHeader 的所有属性
        self.header = header
        return self

    def set_ghost(self, ghost: bool):
        # 设置头部区域的背景颜色为透明
        self.ghost = ghost
        return self

    def set_fixed_header(self, fixed_header: bool):
        # 设置固定 pageHeader 的内容到顶部
        self.fixed_header = fixed_header
        return self

    def set_affix_props(self, affix_props: Any):
        # 设置固钉的配置
        self.affix_props = affix_props
        return self

    def set_footer(self, footer: Any):
        # 设置悬浮在底部的操作栏
        self.footer = footer
        return self

    def set_body(self, body: Any):
        # 设置容器控件里面的内容
        self.body = body
        return self

    def set_water_mark_props(self, water_mark_props: Any):
        # 设置配置水印
        self.water_mark_props = water_mark_props
        return self

    def set_tab_props(self, tab_props: Any):
        # 设置 tab 的配置
        self.tab_props = tab_props
        return self


from typing import Optional

from quark import Request
from quark.services.auth import AuthService


class Export:
    def __init__(self, name: Optional[str] = None):
        self.name = name or "导出"
        self.type = "link"
        self.size = "small"
        self.target = "_blank"
        self.only_on_index_table_row = True

    async def get_href(self, request: Request) -> str:
        token = AuthService(request).get_token()
        return request.url.path.replace("/index", f"/export?id=${{id}}&token={token}")



from ..models.attachment_category import AttachmentCategory


class AttachmentCategoryService:
    def __init__(self):
        pass

    async def get_list(self, admin_id):
        categories = await AttachmentCategory.filter(source="ADMIN", uid=admin_id).all()
        return categories



from typing import Optional

from quark import Request
from quark.template.action import Link


class EditLink(Link):

    def __init__(self, name: Optional[str] = None):
        self.name = name or "编辑"
        self.type = "link"
        self.size = "small"
        self.set_only_on_index_table_row(True)

    async def get_href(self, request: Request) -> str:
        # 替换路径中的 /index 为 /edit&id=${id}
        href = request.url.path.replace("/index", "/edit&id=${id}")
        return f"#/layout/index?api={href}"



from .action import Action
from .drawer import Drawer
from .drawer_form import DrawerForm
from .dropdown import Dropdown
from .link import Link
from .modal import Modal
from .modal_form import ModalForm
from .step import Step
from .switch import Switch



from quark.template.action import Action


class FormExtraBack(Action):

    def __init__(self):

        # 文字
        self.name = "返回上一页"

        # 类型
        self.type = "link"

        # 行为类型
        self.action_type = "back"

        # 在表单页右上角自定义区域展示
        self.set_show_on_form_extra()

        # 在详情页右上角自定义区域展示
        self.set_show_on_detail_extra()



from typing import Any, Dict
from .base import Base


class Date(Base):

    component: str = "dateField"
    """
    组件名称
    """

    allow_clear: bool = True
    """
    是否支持清除
    """

    auto_focus: bool = False
    """
    自动获取焦点
    """

    bordered: bool = True
    """
    是否有边框
    """

    class_name: str = True
    """
    自定义类名
    """

    default_value: Any = None
    """
    默认的选中项
    """

    disabled: Any = None
    """
    禁用状态
    """

    format: str = "YYYY-MM-DD"
    """
    设置日期格式
    """

    popup_class_name: str = None
    """
    额外的弹出日历 className
    """

    input_read_only: bool = False
    """
    设置输入框为只读
    """

    locale: Any = None
    """
    国际化配置
    """

    mode: str = None
    """
    日期面板的状态
    """

    next_icon: Any = None
    """
    自定义下一个图标
    """

    open: bool = None
    """
    控制浮层显隐
    """

    picker: str = None
    """
    设置选择器类型
    """

    placeholder: str = "请选择"
    """
    输入框占位文本
    """

    placement: str = None
    """
    浮层预设位置
    """

    popup_style: Any = None
    """
    额外的弹出日历样式
    """

    prev_icon: Any = None
    """
    自定义上一个图标
    """

    size: str = None
    """
    输入框大小
    """

    status: str = None
    """
    设置校验状态
    """

    style: Dict[str, Any] = None
    """
    自定义样式
    """

    suffix_icon: Any = None
    """
    自定义的选择框后缀图标
    """

    super_next_icon: Any = None
    """
    自定义 << 切换图标
    """

    super_prev_icon: Any = None
    """
    自定义 >> 切换图标
    """

    value: Any = None
    """
    指定选中项
    """

    default_picker_value: str = None
    """
    默认面板日期
    """

    show_now: bool = None
    """
    当设定了 showTime 的时候，面板是否显示“此刻”按钮
    """

    show_time: Any = None
    """
    增加时间选择功能
    """

    show_today: bool = None
    """
    是否展示“今天”按钮
    """

    def set_allow_clear(self, allow_clear: bool):
        """
        设置是否支持清除功能。
        :param allow_clear: 是否支持清除的布尔值
        :return: 当前 Component 实例
        """
        self.allow_clear = allow_clear
        return self

    def set_auto_focus(self, auto_focus: bool):
        """
        设置是否自动获取焦点。
        :param auto_focus: 是否自动获取焦点的布尔值
        :return: 当前 Component 实例
        """
        self.auto_focus = auto_focus
        return self

    def set_bordered(self, bordered: bool):
        """
        设置是否有边框。
        :param bordered: 是否有边框的布尔值
        :return: 当前 Component 实例
        """
        self.bordered = bordered
        return self

    def set_class_name(self, class_name: str):
        """
        设置组件的自定义类名。
        :param class_name: 类名字符串
        :return: 当前 Component 实例
        """
        self.class_name = class_name
        return self

    def set_default_value(self, default_value: Any):
        self.default_value = default_value
        return self

    def set_value(self, value: Any):
        self.value = value
        return self

    def set_format(self, format: str):
        """
        设置日期格式。
        :param format: 日期格式字符串
        :return: 当前 Component 实例
        """
        self.format = format
        return self

    def set_popup_class_name(self, popup_class_name: str):
        """
        设置额外的弹出日历 className。
        :param popup_class_name: 类名字符串
        :return: 当前 Component 实例
        """
        self.popup_class_name = popup_class_name
        return self

    def set_input_read_only(self, input_read_only: bool):
        """
        设置输入框是否为只读。
        :param input_read_only: 是否只读的布尔值
        :return: 当前 Component 实例
        """
        self.input_read_only = input_read_only
        return self

    def set_locale(self, locale: Any):
        """
        设置国际化配置。
        :param locale: 国际化配置数据，可以是字典或其他合适的数据结构
        :return: 当前 Component 实例
        """
        self.locale = locale
        return self

    def set_mode(self, mode: str):
        """
        设置日期面板的状态。
        :param mode: 状态字符串，如 'time'、'date' 等
        :return: 当前 Component 实例
        """
        self.mode = mode
        return self

    def set_next_icon(self, next_icon: Any):
        """
        设置自定义下一个图标。
        :param next_icon: 图标数据，可以是字符串、图片路径等
        :return: 当前 Component 实例
        """
        self.next_icon = next_icon
        return self

    def set_open(self, open: bool):
        """
        控制浮层的显隐。
        :param open: 是否显示浮层的布尔值
        :return: 当前 Component 实例
        """
        self.open = open
        return self

    def set_picker(self, picker: str):
        """
        设置选择器类型。
        :param picker: 选择器类型字符串，如 'date'、'week' 等
        :return: 当前 Component 实例
        """
        self.picker = picker
        return self

    def set_placeholder(self, placeholder: str):
        """
        设置输入框的占位文本。
        :param placeholder: 占位文本字符串
        :return: 当前 Component 实例
        """
        self.placeholder = placeholder
        return self

    def set_placement(self, placement: str):
        """
        设置浮层的预设位置。
        :param placement: 位置字符串，如 'bottomLeft'、'topRight' 等
        :return: 当前 Component 实例
        """
        self.placement = placement
        return self

    def set_popup_style(self, popup_style: Any):
        """
        设置额外的弹出日历样式。
        :param popup_style: 样式数据，可以是字典或其他合适的数据结构
        :return: 当前 Component 实例
        """
        self.popup_style = popup_style
        return self

    def set_prev_icon(self, prev_icon: Any):
        """
        设置自定义上一个图标。
        :param prev_icon: 图标数据，可以是字符串、图片路径等
        :return: 当前 Component 实例
        """
        self.prev_icon = prev_icon
        return self

    def set_size(self, size: str):
        """
        设置输入框的大小。
        :param size: 大小字符串，如 'large'、'middle'、'small'
        :return: 当前 Component 实例
        """
        self.size = size
        return self

    def set_status(self, status: str):
        """
        设置校验状态。
        :param status: 状态字符串，如 'error'、'warning'
        :return: 当前 Component 实例
        """
        self.status = status
        return self

    def set_suffix_icon(self, suffix_icon: Any):
        """
        设置自定义的选择框后缀图标。
        :param suffix_icon: 图标数据，可以是字符串、图片路径等
        :return: 当前 Component 实例
        """
        self.suffix_icon = suffix_icon
        return self

    def set_super_next_icon(self, super_next_icon: Any):
        """
        设置自定义 << 切换图标。
        :param super_next_icon: 图标数据，可以是字符串、图片路径等
        :return: 当前 Component 实例
        """
        self.super_next_icon = super_next_icon
        return self

    def set_super_prev_icon(self, super_prev_icon: Any):
        """
        设置自定义 >> 切换图标。
        :param super_prev_icon: 图标数据，可以是字符串、图片路径等
        :return: 当前 Component 实例
        """
        self.super_prev_icon = super_prev_icon
        return self

    def set_default_picker_value(self, default_picker_value: str):
        """
        设置默认面板日期。
        :param default_picker_value: 日期字符串
        :return: 当前 Component 实例
        """
        self.default_picker_value = default_picker_value
        return self

    def set_show_now(self, show_now: bool):
        """
        当设定了 showTime 的时候，设置面板是否显示“此刻”按钮。
        :param show_now: 是否显示“此刻”按钮的布尔值
        :return: 当前 Component 实例
        """
        self.show_now = show_now
        return self

    def set_show_time(self, show_time: Any):
        """
        增加时间选择功能。
        :param show_time: 时间选择功能相关配置数据，可以是字典或其他合适的数据结构
        :return: 当前 Component 实例
        """
        self.show_time = show_time
        return self

    def set_show_today(self, show_today: bool):
        """
        设置是否展示“今天”按钮。
        :param show_today: 是否展示“今天”按钮的布尔值
        :return: 当前 Component 实例
        """
        self.show_today = show_today
        return self



from typing import Any, Dict, List, Optional

from fastapi import Request

from .action import Action


class Modal(Action):
    """
    表示一个模态框组件，支持配置宽度和关闭时的行为。
    """

    action_type: str = "modal"

    # 弹出层宽度，默认值为 520
    width: int = 520

    # 关闭时销毁弹出层里的子元素，默认为 False
    destroy_on_close: bool = False

    # 设置按钮类型，primary | ghost | dashed | link | text | default
    type: str = "default"

    # 设置按钮大小,large | middle | small | default
    size: str = "small"

    async def content(self, request: Request) -> Optional[Any]:
        """
        获取模态框内容，可以是任意类型的数据
        """
        return None

    async def data(self, request: Request) -> Dict[str, Any]:
        """
        异步获取数据
        """
        return {}

    def get_width(self) -> int:
        """获取弹出层宽度"""
        return self.width

    def get_destroy_on_close(self) -> bool:
        """关闭时销毁 Modal 里的子元素"""
        return self.destroy_on_close

    async def get_body(self, request: Request) -> Optional[Any]:
        """获取弹窗主体内容"""
        return await self.content(request)

    async def get_actions(self, request: Request) -> List[Any]:
        """
        获取弹窗行为按钮列表
        """
        return []



from quark import APIRouter, JSONResponse, Request, jsonable_encoder

from .. import loader

router = APIRouter(prefix="/api/admin", tags=["仪表盘"])


@router.get("/dashboard/{resource}/index")
async def index(request: Request, resource: str):
    res = await loader.load_resource_object(request, resource, "Dashboard")
    return JSONResponse(
        content=jsonable_encoder(await res.render(request), exclude_none=True)
    )



from pydantic import Field, model_validator
from typing import List
from ..component import Component

class Footer(Component):
    component: str = Field(default="footer")
    copyright: str = Field("", description="版权信息")
    links: List[dict] = Field([], description="链接信息")

    @model_validator(mode="after")
    def init(self):
        self.set_key()
        return self

    def set_style(self, style: dict):
        """Set style."""
        self.style = style
        return self

    def set_copyright(self, copyright: str):
        """版权信息"""
        self.copyright = copyright
        return self

    def set_links(self, links: List[dict]):
        """版权信息"""
        self.links = links
        return self


from typing import Any, List, Optional

from pydantic import Field, model_validator

from ..component import Component


class Modal(Component):
    component: Optional[str] = Field(default="modal")
    title: Any = Field(None, description="标题")
    centered: bool = Field(False, description="垂直居中展示 Modal")
    closable: bool = Field(True, description="是否显示右上角的关闭按钮")
    destroy_on_close: bool = Field(
        False, alias="destroyOnClose", description="关闭时销毁 Modal 里的子元素"
    )
    focus_trigger_after_close: bool = Field(
        False,
        alias="focusTriggerAfterClose",
        description="设置按钮形状，可选值为 circle、 round 或者不设",
    )
    keyboard: bool = Field(True, description="是否支持键盘 esc 关闭")
    mask: bool = Field(True, description="是否展示遮罩")
    mask_closable: bool = Field(
        True, alias="maskClosable", description="点击蒙层是否允许关闭"
    )
    open: bool = Field(False, description="对话框是否可见")
    width: int = Field(520, description="宽度")
    z_index: int = Field(1000, alias="zIndex", description="设置 Modal 的 z-index")
    actions: List[Any] = Field([], description="弹窗行为")
    init_api: Optional[Any] = Field(None, alias="initApi", description="数据初始化接口")
    body: Any = Field(None, description="容器控件里面的内容")

    @model_validator(mode="after")
    def init(self):
        self.set_key("modal")
        return self

    def set_style(self, style: Any):
        """Set style."""
        self.style = style
        return self

    def set_title(self, title: Any):
        """标题"""
        self.title = title
        return self

    def set_body(self, body: Any):
        """容器控件里面的内容"""
        self.body = body
        return self

    def set_centered(self, centered: bool):
        """垂直居中展示 Modal"""
        self.centered = centered
        return self

    def set_closable(self, closable: bool):
        """是否显示右上角的关闭按钮"""
        self.closable = closable
        return self

    def set_destroy_on_close(self, destroy_on_close: bool):
        """关闭时销毁 Modal 里的子元素"""
        self.destroy_on_close = destroy_on_close
        return self

    def set_focus_trigger_after_close(self, focus_trigger_after_close: bool):
        """设置按钮形状，可选值为 circle、 round 或者不设"""
        self.focus_trigger_after_close = focus_trigger_after_close
        return self

    def set_keyboard(self, keyboard: bool):
        """是否支持键盘 esc 关闭"""
        self.keyboard = keyboard
        return self

    def set_mask(self, mask: bool):
        """是否展示遮罩"""
        self.mask = mask
        return self

    def set_mask_closable(self, mask_closable: bool):
        """点击蒙层是否允许关闭"""
        self.mask_closable = mask_closable
        return self

    def set_open(self, open_: bool):
        """对话框是否可见"""
        self.open = open_
        return self

    def set_width(self, width: int):
        """宽度"""
        self.width = width
        return self

    def set_z_index(self, z_index: int):
        """设置 Modal 的 z-index"""
        self.z_index = z_index
        return self

    def set_actions(self, actions: List[Any]):
        """弹窗行为"""
        self.actions = actions
        return self

    def set_init_api(self, api: Any):
        """数据初始化接口"""
        self.init_api = api
        return self



from .search import Search


class DatetimeRange(Search):
    """日期组件"""

    component = "datetimeRangeField"

    def __init__(self, column: str = "", name: str = ""):
        self.component = "datetimeRangeField"
        return self



from tortoise import fields
from tortoise.models import Model


class Permission(Model):
    id = fields.IntField(pk=True)
    name = fields.CharField(max_length=500)
    guard_name = fields.CharField(max_length=100, null=True)
    path = fields.CharField(max_length=500)
    method = fields.CharField(max_length=500)
    remark = fields.CharField(max_length=100, null=True)
    created_at = fields.DatetimeField(auto_now_add=True)
    updated_at = fields.DatetimeField(auto_now=True)

    class Meta(Model.Meta):
        table = "permissions"



from ..component import Component
from typing import Any, Optional

class Item(Component):
    """
    表示下拉框中的一个项，包含了各种自定义选项，如标签、操作类型、模态框、抽屉、API 操作和加载状态等。
    """
    component: str = "itemStyle"
    label: str  # 标签
    block: bool = False  # 是否为块级元素
    danger: bool = False  # 是否为危险按钮
    disabled: bool = False  # 是否禁用按钮
    ghost: bool = False  # 是否为幽灵按钮
    icon: str = ""  # 图标
    shape: Optional[str] = None  # 按钮形状，可选值为 "circle"、"round" 或 None
    size: Optional[str] = None  # 按钮大小，可选值为 "large"、"middle"、"small" 或 None
    type: Optional[str] = None  # 按钮类型
    action_type: Optional[str] = None  # 操作类型，例如 "ajax"、"link"
    submit_form: Optional[Any] = None  # 提交的表单 key
    href: Optional[str] = None  # 跳转的 URL
    target: Optional[str] = None  # 跳转的目标（例如 "_blank"）
    modal: Optional[Any] = None  # 模态框组件
    drawer: Optional[Any] = None  # 抽屉组件
    confirm_title: Optional[str] = None  # 确认框的标题
    confirm_text: Optional[str] = None  # 确认框的文本
    confirm_type: Optional[str] = None  # 确认框的类型
    api: Optional[str] = None  # 调用的 API 地址
    reload: Optional[str] = None  # 执行操作后需要刷新的组件
    with_loading: bool = False  # 是否显示加载状态

    def set_label(self, label: str) -> "Item":
        """
        设置按钮的标签。

        Args:
            label (str): 按钮的标签

        Returns:
            Item: 当前实例，标签已更新
        """
        self.label = label
        return self

    def set_block(self, block: bool) -> "Item":
        """
        设置按钮是否为块级元素（占满父容器宽度）。

        Args:
            block (bool): 是否为块级元素

        Returns:
            Item: 当前实例，块级设置已更新
        """
        self.block = block
        return self

    def set_danger(self, danger: bool) -> "Item":
        """
        设置按钮是否为危险按钮。

        Args:
            danger (bool): 是否为危险按钮

        Returns:
            Item: 当前实例，危险按钮设置已更新
        """
        self.danger = danger
        return self

    def set_disabled(self, disabled: bool) -> "Item":
        """
        设置按钮是否禁用。

        Args:
            disabled (bool): 是否禁用

        Returns:
            Item: 当前实例，禁用状态已更新
        """
        self.disabled = disabled
        return self

    def set_ghost(self, ghost: bool) -> "Item":
        """
        设置按钮是否为幽灵按钮（背景透明）。

        Args:
            ghost (bool): 是否为幽灵按钮

        Returns:
            Item: 当前实例，幽灵按钮设置已更新
        """
        self.ghost = ghost
        return self

    def set_icon(self, icon: str) -> "Item":
        """
        设置按钮的图标。

        Args:
            icon (str): 图标名称，例如 "edit"

        Returns:
            Item: 当前实例，图标已更新
        """
        self.icon = f"icon-{icon}"
        return self

    def set_shape(self, shape: str) -> "Item":
        """
        设置按钮的形状。

        Args:
            shape (str): 按钮形状，可能的值为 "circle" 或 "round"

        Returns:
            Item: 当前实例，形状已更新
        """
        self.shape = shape
        return self

    def set_size(self, size: str) -> "Item":
        """
        设置按钮的大小。

        Args:
            size (str): 按钮大小，可能的值为 "large"、"middle" 或 "small"

        Returns:
            Item: 当前实例，大小已更新
        """
        self.size = size
        return self

    def set_action_type(self, action_type: str) -> "Item":
        """
        设置按钮的操作类型（例如 "ajax"、"link"）。

        Args:
            action_type (str): 操作类型

        Returns:
            Item: 当前实例，操作类型已更新
        """
        self.action_type = action_type
        return self

    def set_submit_form(self, form_key: str) -> "Item":
        """
        设置提交的表单 key。

        Args:
            form_key (str): 表单的 key 值

        Returns:
            Item: 当前实例，表单提交设置已更新
        """
        self.submit_form = form_key
        return self

    def set_href(self, href: str) -> "Item":
        """
        设置按钮的跳转地址。

        Args:
            href (str): 跳转的 URL 地址

        Returns:
            Item: 当前实例，跳转地址已更新
        """
        self.href = href
        return self

    def set_target(self, target: str) -> "Item":
        """
        设置跳转链接的目标，类似 a 标签中的 target 属性。

        Args:
            target (str): 跳转目标，例如 "_blank" 打开新标签页

        Returns:
            Item: 当前实例，跳转目标已更新
        """
        self.target = target
        return self

    def set_modal(self, modal: Any) -> "Item":
        """
        设置与按钮关联的模态框。

        Args:
            modal (Any): 模态框组件或回调函数

        Returns:
            Item: 当前实例，模态框已设置
        """
        self.modal = modal
        return self

    def set_drawer(self, drawer: Any) -> "Item":
        """
        设置与按钮关联的抽屉。

        Args:
            drawer (Any): 抽屉组件或回调函数

        Returns:
            Item: 当前实例，抽屉已设置
        """
        self.drawer = drawer
        return self

    def set_with_confirm(self, title: str, text: str, confirm_type: str) -> "Item":
        """
        设置在执行操作前的确认对话框。

        Args:
            title (str): 确认对话框的标题
            text (str): 确认对话框的文本
            confirm_type (str): 确认框的类型（例如 "confirm"）

        Returns:
            Item: 当前实例，确认框已设置
        """
        self.confirm_title = title
        self.confirm_text = text
        self.confirm_type = confirm_type
        return self

    def set_api(self, api: str) -> "Item":
        """
        设置执行操作时调用的 API。

        Args:
            api (str): API 地址

        Returns:
            Item: 当前实例，API 已设置
        """
        self.api = api
        self.action_type = "ajax"  # 设置操作类型为 "ajax"
        return self

    def set_reload(self, reload: str) -> "Item":
        """
        设置执行操作后需要刷新的组件。

        Args:
            reload (str): 刷新的组件

        Returns:
            Item: 当前实例，刷新组件已设置
        """
        self.reload = reload
        return self

    def set_with_loading(self, loading: bool) -> "Item":
        """
        设置是否显示加载状态。

        Args:
            loading (bool): 是否显示加载状态

        Returns:
            Item: 当前实例，加载状态已更新
        """
        self.with_loading = loading
        return self



from tortoise import fields
from tortoise.models import Model


class Position(Model):
    id = fields.IntField(pk=True)
    name = fields.CharField(max_length=500)
    sort = fields.IntField(default=0)
    status = fields.IntField(default=1)
    remark = fields.CharField(max_length=100, null=True)
    created_at = fields.DatetimeField(auto_now_add=True)
    updated_at = fields.DatetimeField(auto_now=True)

    class Meta(Model.Meta):
        table = "positions"

    @staticmethod
    async def seeder():
        """
        插入默认职位数据（如果不存在）
        """
        seeders = [
            {"name": "董事长", "sort": 0, "status": 1},
            {"name": "项目经理", "sort": 0, "status": 1},
            {"name": "普通员工", "sort": 0, "status": 1},
        ]

        for data in seeders:
            exists = await Position.filter(name=data["name"]).first()
            if not exists:
                await Position.create(**data)

    @staticmethod
    async def list():
        """
        获取所有启用状态的职位，返回类似 checkbox.Option 格式的数据
        """
        try:
            positions = await Position.filter(status=1).all()
            return [{"label": p.name, "value": p.id} for p in positions]
        except Exception:
            return []



from typing import Any, List

from tortoise.queryset import QuerySet

from quark import Message, Request

from ..performs_queries import PerformsQueries


class ActionRequest:
    """
    处理资源行为请求
    """

    # 请求对象
    request: Request

    # 资源对象
    resource: Any

    # 查询对象
    query: QuerySet

    # 行为
    actions: List[Any]

    # 字段
    fields: List[Any]

    def __init__(
        self,
        request: Request,
        resource: Any,
        query: QuerySet,
        actions: List[Any] = None,
        fields: List[Any] = None,
    ):
        self.request = request
        self.resource = resource
        self.query = query
        self.actions = actions
        self.fields = fields

    def get_actions(self) -> List[Any]:
        """
        获取当前资源上定义的所有行为（包括资源级行为和字段级行为）。

        Returns:
            list: 返回行为列表。
        """
        actions = []

        # 资源上的行为
        if self.actions:
            actions.extend(self.actions)

        # 字段上的行为
        for field in self.fields:
            component = getattr(field, "component", None)
            if component == "actionField" and hasattr(field, "items"):
                action_items = getattr(field, "items", [])
                if isinstance(action_items, list):
                    actions.extend(action_items)

        return actions

    async def handle(self):
        """
        执行指定的行为。

        Returns:
            JSON 响应：成功或错误信息。
        """
        uri_key = self.request.path_params.get("uriKey")

        # 构建查询
        query = PerformsQueries(self.request).build_action_query(self.query)

        actions = self.get_actions()
        for action in actions:
            action_instance = action
            current_uri_key = action_instance.get_uri_key(action)
            action_type = action_instance.get_action_type()

            if action_type == "dropdown":
                dropdown_actioner = action
                for dropdown_action in dropdown_actioner.get_actions():
                    current_uri_key = dropdown_actioner.get_uri_key(dropdown_action)
                    if uri_key == current_uri_key:
                        # 执行前回调
                        try:
                            await self.resource.before_action(
                                self.request, uri_key, query
                            )
                        except Exception as e:
                            return Message.error(str(e))

                        result = await dropdown_action.handle(self.request, query)

                        # 执行后回调
                        try:
                            await self.resource.after_action(
                                self.request, uri_key, query
                            )
                        except Exception as e:
                            return Message.error(str(e))

                        return result
            else:
                if uri_key == current_uri_key:
                    try:
                        await self.resource.before_action(self.request, uri_key, query)
                    except Exception as e:
                        return Message.error(str(e))

                    result = await action.handle(self.request, query)

                    try:
                        # 执行后回调
                        await self.resource.after_action(self.request, uri_key, query)
                    except Exception as e:
                        return Message.error(str(e))

                    return result

        return Message.error("未找到匹配的行为")

    async def values(self):
        """
        获取行为对应的表单数据。

        Returns:
            JSON 响应：返回行为相关的数据。
        """
        uri_key = self.request.path_params.get("uriKey")
        data = {}
        actions = self.get_actions()

        for action in actions:
            action_instance = action
            current_uri_key = action_instance.get_uri_key(action)
            action_type = action_instance.get_action_type()

            if action_type == "dropdown":
                dropdown_actioner = action
                for dropdown_action in dropdown_actioner.get_actions():
                    current_uri_key = dropdown_actioner.get_uri_key(dropdown_action)
                    if uri_key == current_uri_key:
                        data = await dropdown_action.data(self.request)

            else:
                if uri_key == current_uri_key:
                    data = await action.data(self.request)

        return Message.success("获取成功", data)



from typing import List

from tortoise.queryset import QuerySet

from quark import Message, Request
from quark.template.action import Action


class BatchDelete(Action):

    def __init__(self, name: str = "批量删除"):
        self.name = name
        self.icon = "ant-design:delete-outlined"
        self.type = "primary"
        self.size = "small"
        self.reload = "table"
        self.batch = True
        self.danger = True
        self.ghost = True
        self.with_confirm(
            "确定要删除吗？", "删除后数据将无法恢复，请谨慎操作！", "modal"
        )
        self.set_only_on_index(True)

    def get_api_params(self) -> List[str]:
        return ["id"]

    async def handle(self, request: Request, query: QuerySet):
        try:
            await query.all().delete()
            return Message.success("操作成功")
        except Exception as e:
            return Message.error(str(e))



# message.py
class Message:
    # 状态码常量
    StatusOk = 200  # 执行成功
    StatusUnauthorized = 401  # 未授权
    StatusForbidden = 403  # 无权限
    StatusError = 10001  # 自定义错误信息
    StatusParamError = 10002  # 参数错误

    def __init__(self, code=0, msg="", data=None, url=""):
        self.code = code
        self.msg = msg
        self.data = data
        self.url = url

    def to_dict(self):
        result = {
            'code': self.code,
            'msg': self.msg
        }
        if self.data is not None:
            result['data'] = self.data
        if self.url:
            result['url'] = self.url
        return result

    @classmethod
    def success(cls, message="", data=None):
        """返回正确信息"""
        return cls(
            code=cls.StatusOk,
            msg=message,
            data=data
        )

    @classmethod
    def error(cls, *params):
        """返回错误信息，error("内部服务调用异常") | error("错误", {"title":"标题"})"""
        code = cls.StatusError
        msg = ""
        data = None

        if len(params) == 1:
            msg = params[0]
        elif len(params) == 2:
            msg = params[0]
            data = params[1]

        return cls(
            code=code,
            msg=msg,
            data=data
        )

    @classmethod
    def error_by_code(cls, *params):
        """返回错误信息，error_by_code(10001) | error_by_code(10001, {"title":"标题"})"""
        code = cls.StatusError
        msg = ""
        data = None

        if len(params) == 1:
            code = params[0]
        elif len(params) == 2:
            code = params[0]
            data = params[1]

        msg = cls.get_msg_by_code(code)

        return cls(
            code=code,
            msg=msg,
            data=data
        )

    @classmethod
    def redirect_to(cls, *params):
        """输出模版引擎URL跳转，redirect_to("/home/index") | redirect_to("成功", "/home/index") | redirect_to("失败", "/home/index", 10001)"""
        msg = ""
        url = ""
        code = 200

        if len(params) == 1:
            url = params[0]
        elif len(params) == 2:
            msg = params[0]
            url = params[1]
        elif len(params) >= 3:
            msg = params[0]
            url = params[1]
            code = params[2]

        return cls(
            code=code,
            msg=msg,
            url=url
        )

    @classmethod
    def get_msg_by_code(cls, code):
        """根据code获取错误信息"""
        code_maps = [
            {'code': cls.StatusOk, 'msg': "ok"},
            {'code': cls.StatusError, 'msg': "Internal Server Error"},
            {'code': cls.StatusParamError, 'msg': "Param Error"},
            {'code': cls.StatusUnauthorized, 'msg': "Unauthorized"},
            {'code': cls.StatusForbidden, 'msg': "Forbidden"},
        ]
        
        for code_map in code_maps:
            if code_map['code'] == code:
                return code_map['msg']
        return ""


from pydantic import model_validator
from typing import Any
from ..component import Component


class SubMenu(Component):
    component: str = "menuSubMenu"
    disabled: bool = False
    icon: str = ""
    popup_class_name: str = ""
    popup_offset: Any = None
    title: str = ""
    items: Any = None

    @model_validator(mode="after")
    def init(self):
        self.set_key("DEFAULT_KEY", False)
        return self

    # 是否禁用
    def set_disabled(self, disabled: bool) -> "SubMenu":
        self.disabled = disabled
        return self

    # 菜单图标
    def set_icon(self, icon: str) -> "SubMenu":
        self.icon = icon
        return self

    # 子菜单样式，mode="inline" 时无效
    def set_popup_class_name(self, popup_class_name: str) -> "SubMenu":
        self.popup_class_name = popup_class_name
        return self

    # 子菜单偏移量，mode="inline" 时无效
    def set_popup_offset(self, popup_offset: Any) -> "SubMenu":
        self.popup_offset = popup_offset
        return self

    # 子菜单项值
    def set_title(self, title: str) -> "SubMenu":
        self.title = title
        return self

    # 菜单项
    def set_items(self, items: Any) -> "SubMenu":
        self.items = items
        return self



from ..models.action_log import ActionLog


class ActionLogService:
    def __init__(self):
        pass

    async def insert_get_id(self, data):
        action_log = await ActionLog.create(
            uid=data.uid, url=data.url, ip=data.ip, type=data.type, remark=data.remark
        )
        return action_log.id

    async def count(self):
        return await ActionLog.all().count()



from typing import Any, Dict, List

from tortoise.queryset import QuerySet

from quark import Message, Request
from quark.component.form import field
from quark.template.action import ModalForm


class ModalForm(ModalForm):

    def __init__(self):
        self.name = "Test"
        self.type = "link"
        self.destroy_on_close = True
        self.api_params = ["id"]
        self.set_only_on_index_table_row(True)

    async def fields(self, request: Request) -> List[Dict[str, Any]]:
        return [
            field.id("id", "ID"),
            field.text("username", "用户名"),
        ]

    async def data(self, request: Request) -> Dict[str, Any]:

        id_value = request.query_params.get("id", "")
        return {"id": id_value}

    async def handle(self, request: Request, query: QuerySet):
        return Message.error("method not implemented")
